/******/ (function() { // webpackBootstrap
/******/ 	"use strict";
var __webpack_exports__ = {};
/*!****************************************************!*\
  !*** ./src/js/pro/admin/treweler-manage-import.js ***!
  \****************************************************/


class TrewelerManageImport {
  constructor() {
    this.hideImportUnnecessarySettings();
    if (this.isImportListPage()) return;
    this.initElements();
    this.importPageHandlers();
    this.importValidation();
  }

  isImportListPage() {
    return document.body.classList.contains('edit-php');
  }

  importCanBeRun() {
    const spreadsheetId = this.$spreadsheetId.value;
    const sheetId = this.$sheetId.value;
    const rowsIgnored = parseInt(this.$topRowsIgnored.value);

    const isValidRule = value => {
      return lodash.isEmpty(value) || lodash.isNumber(Number(value)) && !lodash.isNaN(Number(value)) && Number(value) >= 1;
    };

    return spreadsheetId !== '' && sheetId !== '' && isValidRule(rowsIgnored) && rowsIgnored !== 0;
  }

  importValidation() {
    if (this.importCanBeRun()) {
      this.$btnImport.removeAttribute('disabled');
    } else {
      this.$btnImport.setAttribute('disabled', '');
    }
  }

  importPageHandlers() {
    let inputChanged = false;
    this.$spreadsheetId.addEventListener('input', e => {
      if (inputChanged) return false;
      inputChanged = true;
      const $thisInput = e.target;
      const inputVal = $thisInput.value;
      const match = inputVal.match(/spreadsheets\/d\/([a-zA-Z0-9_-]+)/);

      if (match && match[1]) {
        e.target.value = match[1];
      }

      const $spnner = $thisInput.closest('.form-group').getElementsByClassName('spinner')[0];
      $thisInput.setAttribute('readonly', 'true');
      $spnner.classList.add('is-active');
      this.clearNotices();
      const data = new FormData(e.srcElement.form);
      data.append('action', 'twer_get_sheet_ids');
      fetch(twer_ajax_import.url, {
        method: 'POST',
        credentials: 'same-origin',
        body: data
      }).then(response => response.json()).then(data => {
        inputChanged = false;
        console.log(data);

        if (!data.success) {
          this.showNotice(data);
        }

        if (data.success) {
          if (data.data.sheets.length > 0) {
            this.$sheetIdRow.classList.remove('d-none');
            const options = document.createDocumentFragment();

            for (let i = 0; i < data.data.sheets.length; i++) {
              const title = data.data.sheets[i].properties.title;
              const sheetId = data.data.sheets[i].properties.sheetId;
              const option = document.createElement('option');
              option.value = sheetId;
              option.innerHTML = title;
              options.appendChild(option);
            }

            this.clearSelect(this.$sheetId);
            this.$sheetId.appendChild(options);
          } else {
            this.clearSelect(this.$sheetId);
          }
        } else {
          this.clearSelect(this.$sheetId);
          this.$sheetIdRow.classList.add('d-none');
        }

        $spnner.classList.remove('is-active');
        $thisInput.removeAttribute('readonly');
        this.importValidation();
      }).catch(error => {
        console.log('[WP Pageviews Plugin]');
        console.error(error);
      });
    });
    this.$sheetId.addEventListener('change', e => {
      this.importValidation();
    });
    this.$topRowsIgnored.addEventListener('input', e => {
      this.importValidation();
    });
    this.$btnImport.addEventListener('click', e => {
      e.preventDefault();
      const form = e.target.form;
      const data = new FormData(form);
      data.append('action', 'twer_import_markers');
      this.clearNotices();
      this.startLoading();
      this.hideProgressBar();
      this.resetProgressBar();
      fetch(twer_ajax_import.url, {
        method: 'POST',
        credentials: 'same-origin',
        body: data
      }).then(response => response.json()).then(data => {
        console.log(data);
        this.showProgressBar();
        this.updateProgressBar(data);
        this.showNotice(data);
      }).catch(error => {
        console.log('[WP Pageviews Plugin]');
        console.error(error);
      });
      setTimeout(() => {
        this.checkProgressImport(form);
      }, 200);
    });
  }

  checkProgressImport(form) {
    const data = new FormData(form);
    data.append('action', 'twer_get_import_progress');
    fetch(twer_ajax_import.url, {
      method: 'POST',
      credentials: 'same-origin',
      body: data
    }).then(response => response.json()).then(data => {
      //console.log(data);
      if (data.error || !data) {
        this.endLoading();
        this.hideProgressBar();
        this.resetProgressBar();
      } else {
        this.showProgressBar();
        this.updateProgressBar(data);

        if (data.percentage < 100) {
          setTimeout(() => {
            this.checkProgressImport(form);
          }, 200);
        } else {
          this.endLoading();
        }
      }
    }).catch(error => {
      console.log('[WP Pageviews Plugin]');
      console.error(error);
    });
  }

  resetProgressBar() {
    this.updateProgressBar({
      percentage: 0,
      updated: 0,
      imported: 0,
      removed: 0
    });
  }

  updateProgressBarPercent(value) {
    this.$progressBar.style.width = this.$progressBarValue.innerText = `${value}%`;
    this.$progressBar.setAttribute('aria-valuenow', value);
  }

  updateProgressBar(data) {
    this.updateProgressBarPercent(data.percentage);
    this.$updated.innerText = data.updated;
    this.$imported.innerText = data.imported;
    this.$removed.innerText = data.removed;
  }

  showNotice(response) {
    const noticeFragment = document.createDocumentFragment();
    const $notice = document.createElement('div');
    $notice.className = 'notice';
    $notice.id = 'import-notice';
    const $message = document.createElement('p');

    if (response.success) {
      $notice.classList.add('notice-success');
      $message.textContent = response.message;
    } else {
      $notice.classList.add('notice-error');
      $message.textContent = response.error.message;
    }

    $notice.appendChild($message);
    noticeFragment.appendChild($notice);
    document.getElementsByClassName('wp-header-end')[0].after(noticeFragment);
    setTimeout(() => {
      this.clearNotices();
    }, 8000);
  }

  clearNotices() {
    document.querySelectorAll('.wrap div.notice').forEach(notice => {
      notice.remove();
    });
  }

  clearSelect($select) {
    let start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    let reset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    var options = $select.querySelectorAll('option');
    options.forEach((option, index) => {
      if (index <= start) return;
      option.remove();
    });

    if (reset) {
      //$select.options[0].disabled = true;
      //$select.options[0].selected = true;
      $select.options[0].setAttribute('hidden', '');
      $select.options[0].setAttribute('selected', ''); //console.log($select.options[0]);
    }
  }

  initElements() {
    this.$spreadsheetId = document.getElementById('spreadsheet_id');
    this.$btnImport = document.getElementById('js-twer-run-import-markers');
    this.$spinnerImport = document.getElementById('js-twer-import-spinner');
    this.$progressImport = document.getElementById('js-twer-progress-import');
    this.$sheetId = document.getElementById('sheet_id');
    this.$topRowsIgnored = document.getElementById('top_rows_ignored');
    this.$sheetIdRow = document.querySelector('[data-row-id="sheet_id"]');
    this.$progressBar = document.getElementById('js-twer-twer-progressbar');
    this.$progressBarValue = document.getElementById('js-twer-twer-progressbar-value');
    this.$ruleSwitchers = document.getElementsByName('_treweler_removal_rule');
    this.$imported = document.getElementById('js-twer-imported');
    this.$updated = document.getElementById('js-twer-updated');
    this.$removed = document.getElementById('js-twer-removed');
  }

  showProgressBar() {
    this.$progressImport.classList.remove('d-none');
  }

  hideProgressBar() {
    this.$progressImport.classList.add('d-none');
  }

  startLoading() {
    this.$btnImport.setAttribute('disabled', '');
    this.$spinnerImport.classList.add('is-active');
    document.querySelectorAll('.js-twer-form-group').forEach(element => {
      element.classList.add('form-group-loading');
    });
  }

  endLoading() {
    this.$btnImport.removeAttribute('disabled');
    this.$spinnerImport.classList.remove('is-active');
    document.querySelectorAll('.js-twer-form-group').forEach(element => {
      element.classList.remove('form-group-loading');
    });
  }

  hideImportUnnecessarySettings() {
    var _document$getElements, _document$getElements2, _document$getElements3, _document$getElements4, _document$getElementB;

    (_document$getElements = document.getElementsByClassName('inline-edit-password-input')[0]) === null || _document$getElements === void 0 ? void 0 : _document$getElements.closest('.inline-edit-group').remove();
    (_document$getElements2 = document.getElementsByClassName('inline-edit-status')[0]) === null || _document$getElements2 === void 0 ? void 0 : _document$getElements2.remove();
    (_document$getElements3 = document.getElementsByClassName('misc-pub-post-status')[0]) === null || _document$getElements3 === void 0 ? void 0 : _document$getElements3.remove();
    (_document$getElements4 = document.getElementsByClassName('misc-pub-visibility')[0]) === null || _document$getElements4 === void 0 ? void 0 : _document$getElements4.remove();
    (_document$getElementB = document.getElementById('minor-publishing-actions')) === null || _document$getElementB === void 0 ? void 0 : _document$getElementB.remove();
  }

}

window.addEventListener('DOMContentLoaded', event => {
  new TrewelerManageImport();
});
/******/ })()
;
//# sourceMappingURL=treweler-manage-import.js.map