<?php
/**
 * Load PRO assets
 *
 * @package Treweler/Admin
 * @version 1.14
 */
namespace TrewelerPro;


if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( ! class_exists( 'TrewelerPro\TWER_Admin_Assets', false ) ) :

  /**
   * TWER_Admin_Assets Class.
   */
  class TWER_Admin_Assets {

    /**
     * Hook in tabs.
     */
    public function __construct() {
      add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ] );
      add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ] );
      add_action( 'admin_print_scripts', [ $this, 'admin_print_scripts' ] );
    }


    /**
     * Enqueue styles.
     */
    public function admin_styles() {
      $screen = get_current_screen();
      $screen_id = $screen ? $screen->id : '';
      $suffix = defined( 'STYLE_DEBUG' ) && STYLE_DEBUG ? '' : '.min';

      if ( $screen_id === 'twer-custom-fields' || $screen_id === 'twer-templates' || $screen_id === 'twer-shapes' || $screen_id === 'page' || $screen_id === 'treweler-import'  ) {
        wp_enqueue_style( 'treweler-style', TWER()->plugin_url() . '/assets/css/treweler-admin-new' . $suffix . '.css', [], TWER_VERSION );
      }

      if ( $screen_id === 'edit-treweler-import' || $screen_id === 'treweler-import' ) {
        wp_enqueue_style( 'treweler-manage-import-style', TWER()->plugin_url() . '/pro/assets/css/treweler-manage-import' . $suffix . '.css', [], TWER_VERSION);
      }


    }


    public function admin_scripts() {
      $screen = get_current_screen();
      $screen_id = $screen->id ?? '';
      $suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

      $custom_color = get_option( 'treweler_mapbox_colorpicker_custom_color' );
      $defaultColors = '#F44336|#EC407A|#E046C6|#B94AEF|#8559FF|#317DFC|#426D7E|#027F71|#008A43|#238C28|#4B7715|#756B11|#C06018|#9B5A45|#505050|#00B0F6|#00C5AF|#00BC5B|#18AF1F|#5DA900|#A19100|#FF7814|#FF5D28|#FFFFFF|#000000|';


      $wp_localize_script = [
        'url'              => admin_url( 'admin-ajax.php' ),
        'api_key'          => twer_get_api_key(),
        'post_id'          => get_the_ID(),
        'data'             => TWER()->plugin_url() . '/assets/data/',
        'api_key_is_valid' => twer_is_valid_apikey(),
        'fonts_url'        => TWER()->plugin_url() . '/assets/fonts/',
        'strings'          => [
          'unlim' => esc_html__( 'Unlimited', 'treweler' )
        ],
        'custom_colors'    => $custom_color,
        'default_colors'   => $defaultColors
      ];

      if ( $screen_id === 'edit-treweler-import' || $screen_id === 'treweler-import' ) {
        wp_enqueue_script( 'treweler-manage-import-script', TWER()->plugin_url() . '/pro/assets/js/treweler-manage-import' . $suffix . '.js', ['lodash'], TWER_VERSION, true );
        wp_deregister_script( 'autosave' );
        wp_dequeue_script( 'autosave' );

        wp_localize_script( 'treweler-manage-import-script', 'twer_ajax_import', [
          'url'              => admin_url( 'admin-ajax.php' ),
          //'nonce' => wp_create_nonce( 'treweler-import-nonce' ),
        ] );
      }


      if ( $screen_id === 'twer-custom-fields' ) {
        wp_enqueue_script( 'treweler-script', TWER()->plugin_url() . '/pro/assets/js/treweler-custom-fields' . $suffix . '.js', null, TWER_VERSION, true );
        wp_deregister_script( 'autosave' );
        wp_dequeue_script( 'autosave' );
      }

      if ( $screen_id === 'twer-shapes' ) {
        wp_enqueue_script( 'treweler-script', TWER()->plugin_url() . '/pro/assets/js/treweler-draw-shapes' . $suffix . '.js', null, TWER_VERSION, true );

        $maps_posts = get_posts( [
          'post_type'      => 'map',
          'post_status'    => 'publish',
          'posts_per_page' => - 1,
          'orderby'        => 'title',
          'order'          => 'ASC'
        ] );
        $maps = [];
        if ( ! empty( $maps_posts ) ) {
          foreach ( $maps_posts as $map_post ) {
            $id = isset( $map_post->ID ) ? $map_post->ID : 0;
            $map_style = trim( get_post_meta( $id, '_treweler_map_custom_style',
              true ) ) != "" ? get_post_meta( $id, '_treweler_map_custom_style',
              true ) : get_post_meta( $id, '_treweler_map_styles', true );
            $mapProjection = get_post_meta( $id, '_treweler_map_projection', true );

            $mapLightPreset = get_post_meta($id, '_treweler_map_light_preset', true);
            $mapLightPreset = !empty($mapLightPreset) ? $mapLightPreset : 'day';

            $maps[ $id ] = [ 'style' => $map_style, 'projection' => $mapProjection, 'light_preset' => $mapLightPreset ];
          }
        }
        $wp_localize_script['maps'] = $maps;

        global $post;
        $post_id = isset( $post->ID ) ? $post->ID : 0;
        $map_id = get_post_meta( $post_id, '_treweler_map_id', true );


        $current_style_map = twerGetDefaultMapStyle();
        $mapProjection = 'mercator';
        $mapLightPreset = 'day';

        if ( ! empty( $map_id ) ) {
          if ( is_array( $map_id ) ) {
            $current_style_map_id = $map_id[0];
          } else {
            $current_style_map_id = $map_id;
          }

          $current_style_map = trim( get_post_meta( $current_style_map_id, '_treweler_map_custom_style',
            true ) ) != "" ? get_post_meta( $current_style_map_id, '_treweler_map_custom_style',
            true ) : get_post_meta( $current_style_map_id, '_treweler_map_styles', true );

          $mapProjection = get_post_meta( $current_style_map_id, '_treweler_map_projection', true );
          $mapProjection = ! empty( $mapProjection ) ? $mapProjection : 'mercator';


          $mapLightPreset = get_post_meta($current_style_map_id, '_treweler_map_light_preset', true);
          $mapLightPreset = !empty($mapLightPreset) ? $mapLightPreset : 'day';
        }

        $wp_localize_script['current_style_map'] = $current_style_map;
        $wp_localize_script['mapProjection'] = $mapProjection;
        $wp_localize_script['light_preset'] = $mapLightPreset;
      }


      if ( $screen_id === 'twer-templates' ) {
        wp_enqueue_script( 'treweler-script', TWER()->plugin_url() . '/pro/assets/js/treweler-templates' . $suffix . '.js', [ 'jquery', 'jquery-ui-sortable' ], TWER_VERSION, true );
        wp_deregister_script( 'autosave' );
        wp_dequeue_script( 'autosave' );
      }

      if ( $screen_id === 'map' ) {
        wp_enqueue_script( 'treweler-boundaries', TWER()->plugin_url() . '/pro/assets/js/treweler-manage-boundaries' . $suffix . '.js', [ 'treweler-script' ], TWER_VERSION, true );
        wp_enqueue_script( 'treweler-store-locator', TWER()->plugin_url() . '/pro/assets/js/treweler-manage-store-locator' . $suffix . '.js', [ 'treweler-script' ], TWER_VERSION, true );
        wp_enqueue_script( 'treweler-tour', TWER()->plugin_url() . '/pro/assets/js/treweler-manage-tour' . $suffix . '.js', [ 'treweler-script' ], TWER_VERSION, true );
      }


      if ( $screen_id !== 'marker' ) {
        wp_localize_script( 'treweler-script', 'twer_ajax', $wp_localize_script );
      }
    }

    /**
     * Disable auto save posts for custom fields
     *
     * @return void
     */
    public function admin_print_scripts() {
      global $post;
      $post_id = isset( $post->ID ) ? $post->ID : 0;
      if ( get_post_type( $post_id ) === 'twer-custom-fields' || get_post_type( $post_id ) === 'twer-templates'  || get_post_type( $post_id ) === 'treweler-import' ) {
        wp_deregister_script( 'autosave' );
      }
    }

  }

endif;

return new TWER_Admin_Assets();
