<?php
/**
 * Treweler PRO Meta Boxes
 * Sets up the write panels used by maps, routes and markers (custom post types).
 *
 * @package Treweler/Admin/Meta Boxes
 * @version  1.14
 */

namespace TrewelerPro;

defined( 'ABSPATH' ) || exit;

/**
 * TWER_Admin_Meta_Boxes.
 */
class TWER_Admin_Meta_Boxes {

	/**
	 * Constructor.
	 */
	public function __construct() {
		add_action( 'add_meta_boxes', [ $this, 'add_meta_boxes' ], 999 );


		add_action('admin_head', [$this, 'remove_and_add_notices']);
	}

	public function remove_and_add_notices() {
		$current_post_type = get_post_type();
		$current_page = $_GET['page'] ?? null;
		$screen    = get_current_screen();
		$screen_id = $screen ? $screen->id : '';

		$types_to_remove_notices = ['treweler-import', 'twer-custom-fields', 'twer-templates', 'treweler-settings'];

		if (in_array($current_post_type, $types_to_remove_notices, true) || $current_page === 'treweler-settings') {
			remove_all_actions('admin_notices');
			remove_all_actions('all_admin_notices');
		}

		if ($current_post_type === 'treweler-import') {
			add_action('admin_notices', [$this, 'show_admin_message_json_validation']);
		}
		if($screen_id === 'treweler-import') {
			add_action('admin_notices', [$this, 'show_admin_message_spreadsheet_validation']);
		}
	}

	public function show_admin_message_spreadsheet_validation() {
		$postId = get_the_ID();
		$spreadsheetId = twer_get_meta('spreadsheet_id', $postId);
		if(empty($spreadsheetId)) {
			return;
		}
		if ( twerIsSpreadsheetHasSheets( $spreadsheetId ) ) {
			return;
		}
		$sheetIds = twerGetSheetIds( $spreadsheetId );

		if(!$sheetIds['success']) {
			echo '<div class="notice notice-error is-dismissible">';
			echo '<p>'.$sheetIds['error']['message'].'</p>';
			echo '</div>';
		}
	}

	public function show_admin_message_json_validation() {
			if ( twerIsValidGoogleJSONCredentialsFile() ) {
				return;
			}
			echo '<div class="notice notice-warning is-dismissible">';
			echo '<p>';
				printf(__( 'Upload a valid Google Sheets JSON Credentials file on the plugin %ssettings page%s to enable the Import functionality.', 'treweler' ),
					'<a href="' . admin_url( 'admin.php?page=treweler-settings' ) . '" aria-label="' . esc_attr__( 'View Settings settings', 'treweler' ) . '">',
					'</a>'
				);
			echo '</p>';
			echo '</div>';
	}

	/**
	 * Add TWER Meta boxes.
	 */
	public function add_meta_boxes() {
		global $wp_meta_boxes;
		$screen    = get_current_screen();
		$screen_id = $screen ? $screen->id : '';

		if ( twer_is_valid_apikey() ) {
			switch ( $screen_id ) {
				case 'twer-custom-fields' :
					add_meta_box( 'treweler-custom-fields', esc_html__( 'Custom Fields', 'treweler' ), [ new TWER_Meta_Box_Custom_Fields_Details(), 'output' ], $screen_id, 'normal', 'high' );
					break;
				case 'twer-templates' :
					add_meta_box( 'treweler_map_marker_settings_controls-meta', esc_html__( 'Preview Settings', 'treweler' ),
						'TWER_Meta_Box_Marker_Settings::output_template', $screen_id, 'side', 'core' );
					add_meta_box( 'treweler-templates', esc_html__( 'Template Details', 'treweler' ),
						[ new TWER_Meta_Box_Templates_Details(), 'output' ], $screen_id, 'normal', 'high' );
					break;
				case 'twer-shapes' :
					$shape_settings              = new TWER_Meta_Box_Shape_Settings();
					$shape_settings->meta_box_id = 'treweler-shapes-settings';
					$shape_settings->add_meta_box( $shape_settings->meta_box_id, esc_html__( 'Shape Settings', 'treweler' ), 'output', $screen_id, 'side', 'core' );

					$shape_details              = new TWER_Meta_Box_Shape_Details();
					$shape_details->meta_box_id = 'treweler-shapes-details';
					$shape_details->add_meta_box( $shape_details->meta_box_id, esc_html__( 'Shape Details', 'treweler' ), 'output', $screen_id, 'normal', 'high' );
					break;
				case 'treweler-import' :
					$publishBox = $wp_meta_boxes['treweler-import']['side']['core']['submitdiv'];

					unset( $wp_meta_boxes['treweler-import']['side'] );

					$wp_meta_boxes['treweler-import']['side']['core']['submitdiv'] = $publishBox;

					unset( $wp_meta_boxes['treweler-import']['normal'], $wp_meta_boxes['treweler-import']['advanced'] );

					add_meta_box( 'treweler-import-details', esc_html__( 'Import Details', 'treweler' ), [ new TWER_Meta_Box_Import_Details(), 'output' ], $screen_id, 'normal', 'high' );
					break;
			}
		}
	}
}

new TWER_Admin_Meta_Boxes();
