<?php
/**
 * Treweler PRO Admin
 *
 * @class    TWER_Admin
 * @package  Treweler/Admin
 * @version  1.14
 */

namespace TrewelerPro;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}


/**
 * TWER_Admin class.
 */
class TWER_Admin {

	/**
	 * Constructor.
	 */
	public function __construct() {

		add_action( 'init', [ $this, 'includes' ] );

		add_filter( 'twerMainMenuOrder', [ $this, 'setTrewelerMenuOrder' ], 5, 2 );
		add_action( 'twerAdminMenu', [ $this, 'addItemToAdminMenu' ], 5 );
		add_action( 'parent_file', [ $this, 'menuHighlight' ] );
		add_action( 'admin_init', [ $this, 'removeRevisions' ] );


		add_filter( 'twerSetScreenIdsUsedMapbox', [ $this, 'screenIdsUsedMapbox' ] );
		add_filter( 'twerSetScreenIdsUsedMaterialIcons', [ $this, 'screenIdsUsedMaterialIcons' ] );


		add_action( 'admin_init', [ $this, 'register_settings' ] );
		add_action( 'twerSettingsValidation', [ $this, 'validateSettings' ] );
	}

	public function validateSettings( $input ) {
		if ( empty( $input['google_sheets_api_json'] ) ) {
			return;
		}

		$attachment_id = $input['google_sheets_api_json'];
		$file_path     = get_attached_file( $attachment_id );

		if ( $file_path ) {
			$json_string = file_get_contents( $file_path );
			$data        = json_decode( $json_string );

			if ( json_last_error() === JSON_ERROR_NONE ) {
				if ( ! isset( $data->type, $data->private_key_id, $data->private_key, $data->client_email, $data->client_id ) ) {
					add_settings_error(
						'treweler-options', 'treweler_error',
						__( 'Invalid service account JSON file: Missing Required keys', 'treweler' )
					);
				} else {
					$client = new \Google_Client();
					try {
						$client->setAuthConfig($file_path);
					} catch (\Google_Service_Exception $e) {
						add_settings_error(
							'treweler-options', 'treweler_error',
							sprintf(__( 'Invalid service account JSON file: %s', 'treweler' ),  $e->getMessage())
						);
					}
                }
			} else {
				add_settings_error(
					'treweler-options', 'treweler_error',
					__( 'The Google Sheets JSON Credentials data is invalid', 'treweler' )
				);
			}
		}
	}

	public function register_settings() {

		add_settings_field(
			'google_sheets_api_json',
			esc_html__( 'Google Sheets Credentials ', 'treweler' ),
			[ $this, 'google_sheets_api_json_cb' ],
			'treweler-settings',
			'treweler-section',
			[
				'label_for' => 'google_sheets_api_json'
			]
		);
	}


	public function google_sheets_api_json_cb() {
		$options = get_option( 'treweler' );

		$attachment_id = isset( $options['google_sheets_api_json'] ) ? esc_attr( $options['google_sheets_api_json'] ) : '';
		?>
        <div class="form-row align-items-stretch">
            <div class="form-group col">

                <div class="twer-attach js-twer-attach-wrap" data-library-type="application/json" data-button-text="<?php esc_html_e( 'Upload File', 'treweler' ) ?>" data-title="<?php esc_html_e( 'Select JSON File', 'treweler' ); ?>">

                    <div class="twer-attach__thumb js-twer-attach-thumb">
						<?php if ( ! empty( $attachment_id ) ) { ?>
                            <div class="twer-app-file" style="white-space: nowrap;">
								<?php
								$attachment_url = wp_get_attachment_url( $attachment_id );
                                echo $attachment_url ? basename( $attachment_url ) : esc_html__( 'Attachment not found', 'treweler' );
								?>
                            </div>
						<?php } ?>
                        <button type="button" class="btn btn-outline-light twer-attach__add-media js-twer-attach-add" style="<?php echo ! empty( $attachment_id ) ? 'display:none;' : 'display:block'; ?>"><?php _e('Select JSON File','treweler'); ?></button>
                        <input type="hidden" name="treweler[google_sheets_api_json]" id="google_sheets_api_json" value="<?php echo $attachment_id; ?>">
                    </div>

                    <div class="twer-attach__actions js-twer-attach-actions" style="<?php echo ! empty( $attachment_id ) ? 'display:block;' : 'display:none'; ?>">
                        <button type="button" class="button js-twer-attach-remove"><?php esc_html_e( 'Remove', 'treweler' ); ?></button>
                        <button type="button" class="button js-twer-attach-add"><?php esc_html_e( 'Change', 'treweler' ); ?></button>
                    </div>
                </div>


            </div>

        </div>
		<?php


		echo '<p style="font-size: 14px; margin-bottom: 0; margin-top: 5px;" class="description">';
		echo sprintf( '%s %s%s%s %s %s%s%s %s',
			esc_html__( 'Upload a valid', 'treweler' ),
			'<a  style="text-decoration: underline;" href="https://www.youtube.com/watch?v=k_Y4aiz1nt8" target="_blank">',
			esc_html__( 'Google Sheets API JSON File', 'treweler' ),
			'</a>',
			esc_html__( 'to enable the', 'treweler' ),
			'<a style="text-decoration: underline;" href="' . admin_url( 'edit.php?post_type=treweler-import' ) . '" target="_blank">',
			esc_html__( 'Import', 'treweler' ),
			'</a>',
			esc_html__( 'functionality.', 'treweler' ),
		);
		echo '</p>';
	}


	public function screenIdsUsedMapbox( $screenIds ) {
		return wp_parse_args( [
			'twer-templates',
			'twer-shapes'
		], $screenIds );
	}

	public function screenIdsUsedMaterialIcons( $screenIds ) {
		return wp_parse_args( [
			'twer-templates'
		], $screenIds );
	}


	/**
	 * Include any classes we need within admin.
	 */
	public function includes() {
		include_once dirname( __FILE__ ) . '/twer-admin-functions.php';
		include_once dirname( __FILE__ ) . '/class-twer-admin-post-types.php';
		include_once dirname( __FILE__ ) . '/class-twer-admin-assets.php';
	}

	public function setTrewelerMenuOrder( $order, $submenu ) {
		/**
		 * Pro
		 * 0 - Treweler
		 * 1 - Categories
		 * 2 - Shapes
		 * 3 - Custom Fields
		 * 4 - Templates
		 * 5 - Import
		 * 6 - Maps
		 * 7 - Markers
		 * 8 - Routes
		 * 9 - Settings
		 * Maps
		 * Markers
		 * Routes
		 * Shapes
		 * Categories
		 * Custom Fields
		 * Templates
		 * Import
		 * Settings
		 * */

		return [ 0, 6, 7, 8, 2, 1, 3, 4, 5, 9 ];
	}

	/**
	 * Remove revisions for Custom Post Types
	 *
	 * @return void
	 */
	public function removeRevisions() {
		global $wpdb;
		remove_post_type_support( 'twer-custom-fields', 'revisions' );
		remove_post_type_support( 'treweler-import', 'revisions' );
		remove_post_type_support( 'twer-templates', 'revisions' );

		// This query will delete all revisions from your Database table.
		$revision_ids = $wpdb->get_col( $wpdb->prepare( "SELECT `ID` FROM $wpdb->posts WHERE `post_type` = %s AND `post_type` = %s AND `post_type` = %s", 'twer-custom-fields', 'twer-templates', 'treweler-import' ) );

		if ( ! empty( $revision_ids ) ) {
			foreach ( $revision_ids as $revision_id ) {
				wp_delete_post_revision( $revision_id );
			}
		}
	}

	public function addItemToAdminMenu() {
		add_submenu_page( 'treweler', __( 'Categories', 'treweler' ), __( 'Categories', 'treweler' ), 'edit_treweler', 'edit-tags.php?taxonomy=map-category&post_type=map' );
	}


	/**
	 * Highlights the correct top level admin menu item for post type add screens.
	 */
	public function menuHighlight( $parent_file ) {
		global $plugin_page, $submenu_file, $post_type, $taxonomy;
		if ( ( 'map' === $post_type ) && $taxonomy === 'map-category' ) {
			$plugin_page  = 'edit-tags.php?taxonomy=map-category&post_type=map'; // the submenu slug
			$submenu_file = 'edit-tags.php?taxonomy=map-category&post_type=map';    // the submenu slug
		}

		return $parent_file;
	}


}

return new TWER_Admin();
