<?php
/**
 * Custom Fields Details
 * Display the custom fields settings details meta box.
 *
 * @author      Aisconverse
 * @category    Admin
 * @package     Treweler/Admin/Meta Boxes
 * @version     1.05
 */
namespace TrewelerPro;

use TWER_Meta_Boxes;

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Meta_Box_Custom_Fields_Details', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Meta_Boxes', false ) ) {
  include_once TWER_ABSPATH . 'includes/admin/meta-boxes/abstract-class-twer-meta-boxes.php';
}

/**
 * TWER_Meta_Box_Custom_Fields_Details Class.
 */
class TWER_Meta_Box_Custom_Fields_Details extends TWER_Meta_Boxes {

  protected $metabox = 'twer-custom-fields';

  protected function setNestedFields() {

  }

  protected function setNestedTabs() {
  }

  protected function set_tabs() {
  }

  protected function set_fields() {
    $this->fields = [
      [
        'type'     => 'select',
        'name'     => 'custom_field_type',
        'class'    => [ 'js-custom-fields-type' ],
        'label'    => esc_html__( 'Field Type', 'treweler' ),
        'options'  => [
          'text'        => esc_html__( 'Text & Title', 'treweler' ),
          'link'        => esc_html__( 'Link', 'treweler' ),
          'email'       => esc_html__( 'Email', 'treweler' ),
          'phone'       => esc_html__( 'Phone', 'treweler' ),
          'separator'   => esc_html__( 'Separator', 'treweler' ),
          'button'      => esc_html__( 'Button', 'treweler' ),
          'line'        => esc_html__( 'Line', 'treweler' ),
          'number'      => esc_html__( 'Number', 'treweler' ),
          'true_false'  => esc_html__( 'True/False', 'treweler' ),
          'multiselect' => esc_html__( 'Multiselect', 'treweler' ),
          'html' => esc_html__( 'Custom HTML', 'treweler' ),
          'category' =>  esc_html__( 'Categories', 'treweler' ),
        ],
        'selected' => 'text'
      ],
      [
        'type'  => 'text',
        'name'  => 'custom_field_key',
        'atts'  => [ 'readonly', 'onfocus="this.select();"', 'autocomplete="off"' ],
        'label' => esc_html__( 'Field Key', 'treweler' ),
        'value' => $this->unique_field_key
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'custom_field_show',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Show Field Name', 'treweler' ),
        'checked'     => false
      ],



      [
        'type'        => 'text',
        'row_id'      => 'custom_field_text_old',
        'name'        => $this->unique_field_key,
        'atts'        => [ 'disabled' ],
        'class' => [ 'js-twer-old-text-input' ],
        'label'       => esc_html__( 'Default Value', 'treweler' ),
        'value'       => ''
      ],

      [
        'type'   => 'group',
        'row_id' => 'custom_field_text',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'labelThClass' => 'flex-column align-items-start',
        'group'  => [
          [
            'type'        => 'textarea',
            'name'        => 'text',
            'class' => [ 'js-twer-new-text-input' ],
            'atts'        => [ 'cols="60"', 'rows="4"', 'disabled' ],
            'group_class' => 'form-group col-auto',
            'value'       => ''
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_text',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Align', 'treweler' ),
        'group'  => [
          [
            'type'        => 'select',
            'name'        => 'align',
            'options'  => [
              'left'        => esc_html__( 'Left', 'treweler' ),
              'center'        => esc_html__( 'Center', 'treweler' ),
              'right'        => esc_html__( 'Right', 'treweler' ),
            ],
            'selected' => 'left'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_text',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Style', 'treweler' ),
        'group'  => [
          [
            'type'        => 'number',
            'append'      => 'px',
            'group_class' => [ 'col-220' ],
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'value'       => '',
            'placeholder' => '14'
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-220' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_text',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Margins', 'treweler' ),
        'group'  => [
          [
            'type'         => 'number',
            'name'         => 'marginTop',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '10',
            'group_class' => [ 'col-220' ],
            'label_inline' => esc_html__( 'Top', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'marginBottom',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '10',
            'group_class' => [ 'col-220' ],
            'label_inline' => esc_html__( 'Bottom', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],


      [
        'type'   => 'group',
        'row_id' => 'custom_field_text',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Display as Title', 'treweler' ),
        'group'  => [
          [
            'type'        => 'checkbox',
            'name'        => 'showTitle',
            'group_class' => 'form-group col-auto',
            'checked'     => false,
          ]
        ]
      ],







      [
        'type'   => 'group',
        'row_id' => 'custom_field_link',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Align', 'treweler' ),
        'group'  => [
          [
            'type'        => 'select',
            'name'        => 'url_align',
            'options'  => [
              'left'        => esc_html__( 'Left', 'treweler' ),
              'center'        => esc_html__( 'Center', 'treweler' ),
              'right'        => esc_html__( 'Right', 'treweler' ),
            ],
            'selected' => 'left'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_link',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'text',
            'name'        => 'url_label',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Text', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'        => 'url',
            'name'        => 'url',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'URL', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'target',
            'group_class'  => 'form-group col-auto',
            'atts'         => [ 'disabled' ],
            'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
            'checked'      => false
          ],
        ]
      ],



      [
        'type'   => 'group',
        'row_id' => 'custom_field_email',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Align', 'treweler' ),
        'group'  => [
          [
            'type'        => 'select',
            'name'        => 'email_align',
            'options'  => [
              'left'        => esc_html__( 'Left', 'treweler' ),
              'center'        => esc_html__( 'Center', 'treweler' ),
              'right'        => esc_html__( 'Right', 'treweler' ),
            ],
            'selected' => 'left'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_email',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'text',
            'name'        => 'email_label',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Text', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'        => 'email',
            'name'        => 'email',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Email', 'treweler' ),
            'value'       => '',
          ]
        ]
      ],



      [
        'type'   => 'group',
        'row_id' => 'custom_field_phone',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Align', 'treweler' ),
        'group'  => [
          [
            'type'        => 'select',
            'name'        => 'tel_align',
            'options'  => [
              'left'        => esc_html__( 'Left', 'treweler' ),
              'center'        => esc_html__( 'Center', 'treweler' ),
              'right'        => esc_html__( 'Right', 'treweler' ),
            ],
            'selected' => 'left'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_phone',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'text',
            'name'        => 'tel_label',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Text', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'        => 'tel',
            'name'        => 'tel',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Phone Number', 'treweler' ),
            'value'       => '',
          ]
        ]
      ],
      [
        'type'        => 'number',
        'row_id'      => 'custom_field_separator',
        'name'        => $this->unique_field_key,
        'atts'        => [ 'min="0"', 'step="1"', 'disabled' ],
        'append'      => 'px',
        'placeholder' => '0',
        'group_class' => [ 'col-130' ],
        'label'       => esc_html__( 'Default Value', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_button',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'text',
            'name'        => 'btn_label',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Text', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'        => 'url',
            'name'        => 'btn_url',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'URL', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'btn_target',
            'group_class'  => 'form-group col-auto',
            'atts'         => [ 'disabled' ],
            'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
            'checked'      => false
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_button',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Fullwidth Button', 'treweler' ),
        'group'  => [
          [
            'type'        => 'checkbox',
            'name'        => 'btn_fullwidth',
            'group_class' => 'form-group col-auto',
            'checked'     => false,
          ]
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_button',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Background Color', 'treweler' ),
        'group'  => [
          [
            'type'        => 'colorpicker',
            'name'        => 'btn_bg_color',
            'value'       => '#000000',
            'id'          => 'buttonBgColor',
            'group_class' => 'form-group col-auto',
          ]
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_button',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Color', 'treweler' ),
        'group'  => [
          [
            'type'        => 'colorpicker',
            'name'        => 'btn_text_color',
            'value'       => '#ffffff',
            'id'          => 'buttonTextColor',
            'group_class' => 'form-group col-auto',
          ]
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_line',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Margins', 'treweler' ),
        'group'  => [
          [
            'type'         => 'number',
            'name'         => 'margin_top',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '17',
            'label_inline' => esc_html__( 'Top', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'margin_bottom',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '17',
            'label_inline' => esc_html__( 'Bottom', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_line',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Line Color', 'treweler' ),
        'group'  => [
          [
            'type'        => 'colorpicker',
            'name'        => 'line_color',
            'value'       => '#DADADA',
            'id'          => 'lineColor',
            'group_class' => 'form-group col-auto',
          ]
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_number',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'  => 'number',
            'name'  => 'number',
            'atts'  => [ 'step="any"', 'disabled' ],
            'value' => ''
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_number',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Prefix', 'treweler' ),
        'group'  => [
          [
            'type'  => 'text',
            'name'  => 'prefix',
            'atts'  => [ 'disabled' ],
            'value' => '',
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_number',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Suffix', 'treweler' ),
        'group'  => [
          [
            'type'  => 'text',
            'name'  => 'suffix',
            'atts'  => [ 'disabled' ],
            'value' => '',
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_true_false',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'checkbox',
            'name'        => 'true_false',
            'group_class' => 'form-group col-auto',
            'atts'        => [ 'disabled' ],
            'checked'     => true
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_multiselect',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Align', 'treweler' ),
        'group'  => [
          [
            'type'        => 'select',
            'name'        => 'multiselect_align',
            'options'  => [
              'left'        => esc_html__( 'Left', 'treweler' ),
              'center'        => esc_html__( 'Center', 'treweler' ),
              'right'        => esc_html__( 'Right', 'treweler' ),
            ],
            'selected' => 'left'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_multiselect',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Options', 'treweler' ),
        'labelThDescription' => esc_html__( 'Enter each option on a new line. Put the default options in “quotes”.', 'treweler' ),
        'labelThClass' => 'flex-column align-items-start',
        'group'  => [
          [
            'type'        => 'textarea',
            'name'        => 'multiselect',
            'atts'        => [ 'cols="60"', 'rows="8"', 'disabled' ],
            'group_class' => 'form-group col-auto',
            'value'       => ''
          ],
        ]
      ],


      [
        'type'   => 'group',
        'row_id' => 'custom_field_html',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'labelThClass' => 'flex-column align-items-start',
        'group'  => [
          [
            'type'        => 'textarea',
            'name'        => 'html',
            'atts'        => [ 'cols="70"', 'rows="13"', 'disabled' ],
            'group_class' => 'form-group col-auto',
            'value'       => '',
            'bottom_note' => esc_html__('Use [audio mp3="https://example.com/example.mp3"][/audio] shortcode to display the audio player.', 'treweler')
          ],
        ]
      ],

      [
        'type'   => 'group',
        'row_id' => 'custom_field_html',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Margins', 'treweler' ),
        'group'  => [
          [
            'type'         => 'number',
            'name'         => 'marginTop',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '10',
            'group_class' => [ 'col-220' ],
            'label_inline' => esc_html__( 'Top', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'marginBottom',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '10',
            'group_class' => [ 'col-220' ],
            'label_inline' => esc_html__( 'Bottom', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],


      [
        'type'   => 'group',
        'row_id' => 'custom_field_category',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Category Styles', 'treweler' ),
        'group'  => [
          [
            'type'        => 'select',
            'name'        => 'style',
            'class'    => [ 'js-category-style' ],
            'options'  => [
              'text'        => esc_html__( 'Text', 'treweler' ),
              'tags'        => esc_html__( 'Tags', 'treweler' ),
            ],
            'selected' => 'text'
          ],
        ]
      ],

      [
        'type'   => 'group',
        'row_id' => 'custom_field_category',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Style', 'treweler' ),
        'group'  => [
          [
            'type'        => 'number',
            'append'      => 'px',
            'group_class' => [ 'col-220' ],
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'value'       => '',
            'placeholder' => '14'
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-220' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_category',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Text Align', 'treweler' ),
        'group'  => [
          [
            'type'        => 'select',
            'name'        => 'category_align',
            'options'  => [
              'left'        => esc_html__( 'Left', 'treweler' ),
              'center'        => esc_html__( 'Center', 'treweler' ),
              'right'        => esc_html__( 'Right', 'treweler' ),
            ],
            'selected' => 'left'
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_category',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Margins', 'treweler' ),
        'group'  => [
          [
            'type'         => 'number',
            'name'         => 'marginTop',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '10',
            'group_class' => [ 'col-220' ],
            'label_inline' => esc_html__( 'Top', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'marginBottom',
            'atts'         => [ 'step="1"', 'disabled' ],
            'append'       => 'px',
            'placeholder'  => '10',
            'group_class' => [ 'col-220' ],
            'label_inline' => esc_html__( 'Bottom', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],

      [
        'type'        => 'number',
        'name'        => 'custom_field_name_width',
        'label'       => esc_html__( 'Field Name Width', 'treweler' ),
        'append'      => '%',
        'class'    => [ 'js-field-name-width' ],
        'group_class' => [ 'col-220' ],
        'atts'        => [ 'min="0"', 'max="100"', 'step="1"' ],
        'value'       => '',
        'placeholder' => '35'
      ],

    ];
  }


  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct();

    add_filter( 'twer-custom-fields_assign_fields', [ $this, 'toggle_rows' ], 10, 2 );
  }


  public function toggle_rows( $fields, $metabox ) {
    $meta = twer_get_data();

    $field_type = isset( $meta->custom_field_type ) ? $meta->custom_field_type : 'text';

    foreach ( $fields as &$field ) {
      if ( ( 'separator' === $field_type && $field['id'] === 'custom_field_show' ) ||
           ( 'button' === $field_type && $field['id'] === 'custom_field_show' ) ||
           ( 'line' === $field_type && $field['id'] === 'custom_field_show' ) ||
           ( 'true_false' === $field_type && $field['id'] === 'custom_field_show' ) ||

           ( 'link' !== $field_type && 'phone' !== $field_type && 'email' !== $field_type && 'text' !== $field_type && 'number' !== $field_type && $field['id'] === 'custom_field_name_width' )

      ) {
        $field['row_class'] = implode( ' ', array_merge( [ $field['row_class'] ], [ 'd-none' ] ) );
      }

      if ( $field['id'] === 'custom_field_type' || $field['id'] === 'custom_field_key' || $field['id'] === 'custom_field_show' || $field['id'] === 'custom_field_name_width' ) {
        continue;
      }

      if ( 'custom_field_' . $field_type !== $field['id'] ) {
        $field['row_class'] = implode( ' ', array_merge( [ $field['row_class'] ], [ 'd-none' ] ) );
      } else {
        if ( in_array( 'disabled', $field['atts'] ) ) {
          if ( ( $key = array_search( 'disabled', $field['atts'] ) ) !== false ) {
            unset( $field['atts'][ $key ] );
          }
        }

        if ( 'group' === $field['type'] ) {
          foreach ( $field['group'] as $key_group => $group ) {
            if ( in_array( 'disabled', $group['atts'] ) ) {
              if ( ( $key = array_search( 'disabled', $group['atts'] ) ) !== false ) {
                unset( $field['group'][ $key_group ]['atts'][ $key ] );
              }
            }
          }
        }
      }
    }

    return $fields;
  }
}
