<?php
/**
 * Import Details
 * Display the import settings details meta box.
 *
 * @author      Aisconverse
 * @category    Admin
 * @package     Treweler/Admin/Meta Boxes
 * @version     1.14
 */

namespace TrewelerPro;

use TWER_Meta_Boxes;

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Meta_Box_Import_Details', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Meta_Boxes', false ) ) {
  include_once TWER_ABSPATH . 'includes/admin/meta-boxes/abstract-class-twer-meta-boxes.php';
}

/**
 * TWER_Meta_Box_Import_Details Class.
 */
class TWER_Meta_Box_Import_Details extends TWER_Meta_Boxes {

  protected $metabox = 'treweler-import';

  protected function setNestedFields() {

  }

  protected function setNestedTabs() {
  }

  protected function set_tabs() {
  }

  protected function set_fields() {

    $templateNote = esc_html__( 'Verify the spreadsheet\'s %1$ssharing settings%3$s. Follow the plugin documentation and %2$swell-filled example%3$s to fill the spreadsheet out correctly.', 'treweler' );

//    $sharing_settings_link = '<a href="url_to_sharing_settings"></a>';
//    $well_filled_example_link = '<a href="url_to_example">well-filled example</a>';
//
//    $result = sprintf($template, $sharing_settings_link, $well_filled_example_link);


    $this->fields = [
      [
        'type'        => 'text',
        'name'        => 'spreadsheet_id',
        'group_class' => [ 'col-416 align-items-center justify-content-end d-flex js-twer-form-group' ],
        'atts' => twerIsValidGoogleJSONCredentialsFile() ? [] : ['disabled'],
        'spinner'     => true,
        'class'       => [ 'pr-4' ],
        'label'       => esc_html__( 'Spreadsheet URL or ID', 'treweler' ),
        'placeholder' => esc_html__( 'Google Spreadsheet ID', 'treweler' ),
        'value'       => '',
        'bottom_note' => sprintf( $templateNote, '<a style="text-decoration: underline;" href="https://www.youtube.com/watch?v=21aO8QZgMTw" target="_blank">', '<a style="text-decoration: underline;" href="https://docs.google.com/spreadsheets/d/1CzBgpYWMyvVzj8QbPJbSXwhF11whZrJOCwfqgN-xiWQ/edit?usp=sharing" target="_blank">', '</a>' )
      ],
      [
        'type'          => 'select',
        'name'          => 'sheet_id',
        'atts' => twerIsValidGoogleJSONCredentialsFile() ? [] : ['disabled'],
        'group_class'   => [ 'col-416 js-twer-form-group' ],
        'label'         => esc_html__( 'Sheet ID', 'treweler' ),
        'option_hidden' => 0,
        'options'       => $this->getHTMLSheetIDOptions(),
        'selected'      => ''
      ],
      [
        'type'        => 'number',
        'name'        => 'top_rows_ignored',
        'label'       => esc_html__( 'Number of Top Rows Ignored', 'treweler' ),
        'placeholder' => '1',
        'atts'        => twerIsValidGoogleJSONCredentialsFile() ? [ 'min="1"', 'step="1"' ] : ['disabled'],
        'group_class' => [ 'col-93 js-twer-form-group' ],
        'value'       => '',
      ],
      [
        'type'         => 'radios',
        'name'         => 'removal_rule',
        'group_class'  => 'form-group col-auto js-twer-form-group',
        'label'        => esc_html__( 'Removal Rule', 'treweler' ),
        'atts' => twerIsValidGoogleJSONCredentialsFile() ? [] : ['disabled'],
        'options'      => [
          'dont_rm_markers'                => esc_html__( 'Don\'t remove markers.', 'treweler' ),
          'rm_markers_missing_table'       => esc_html__( 'Remove markers that were added with this import but are not in this Google Spreadsheet.', 'treweler' ),
          'rm_markers_missing_table_early' => esc_html__( 'Remove any markers that are not in this Google Spreadsheet.', 'treweler' ),
        ],
        'selected'     => 'dont_rm_markers',
        'separator'    => '',
        'label_ids'    => true,
        'labelThClass' => 'flex-column align-items-baseline',
      ],
      [
        'type'         => 'html',
        'name'         => '',
        'labelThClass' => 'pt-0 pb-0',
        'label'        => '',
        'group_class'  => [ 'col-416' ],
        'value'        => $this->getHTMLImportElements()
      ],
    ];
  }

  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct();

    add_filter( 'treweler-import_assign_fields', [ $this, 'toggle_rows' ], 10, 2 );
  }

  public function toggle_rows( $fields, $metabox ) {
    $meta = twer_get_data();

    $spreadsheetId = isset( $meta->spreadsheet_id ) ? (string) $meta->spreadsheet_id : '';

    if ( twerIsSpreadsheetHasSheets( $spreadsheetId ) ) {
      return $fields;
    }

	  foreach ( $fields as &$field ) {
		  if ( $field['id'] === 'sheet_id' ) {
			  //if ( empty( $meta->sheet_id ) || empty( $spreadsheetId ) ) {
				  $field['row_class'] = implode( ' ', array_merge( [ $field['row_class'] ], [ 'd-none' ] ) );
			  //}
		  }
	  }

    return $fields;
  }


  protected function getHTMLSheetIDOptions(): array {
    global $post;
    $post_id = $post->ID ?? 0;

    $options = [
      '' => esc_html__( 'Choose Your Sheet ID', 'treweler' )
    ];

    $spreadsheetId = twer_get_meta( 'spreadsheet_id', $post_id );
    if ( twerIsSpreadsheetHasSheets( $spreadsheetId ) ) {
      $sheetIds = twerGetSheetIds( $spreadsheetId );
      if ( ! empty( $sheetIds['data']->sheets ) ) {
        foreach ( $sheetIds['data']->sheets as $sheet ) {
          $options[ $sheet->properties->sheetId ] = esc_html( $sheet->properties->title );
        }
      }
    }

    return $options;
  }

  protected function getHTMLImportElements() {
	  ob_start();

	  ?>
      <div class="row align-items-center">
          <div class="col-auto">
              <button type="button" disabled id="js-twer-run-import-markers" class="btn btn-sm btn-primary"><?php _e( 'Run Import', 'treweler' ); ?></button>
          </div>
          <div class="col-auto pl-0"><span id="js-twer-import-spinner" class="spinner m-0"></span></div>
      </div>

      <div id="js-twer-progress-import" class="d-none mt-3">
          <div class="row align-items-center mb-1">
              <div class="col">
                  <div class="progress">
                      <div id="js-twer-twer-progressbar" class="progress-bar bg-success" role="progressbar" style="width: 0%" aria-valuenow="25" aria-valuemin="0" aria-valuemax="100"></div>
                  </div>
              </div>
              <div id="js-twer-twer-progressbar-value" class="col-auto pl-0">0%</div>
          </div>

          <div class="progress-status">
              <span id="js-twer-imported">0</span>&nbsp;<?php esc_html_e( 'Imported', 'treweler' ); ?>,
              <span id="js-twer-updated">0</span>&nbsp;<?php esc_html_e( 'Updated', 'treweler' ); ?>,
              <span id="js-twer-removed">0</span>&nbsp;<?php esc_html_e( 'Removed', 'treweler' ); ?>
          </div>
      </div>
    <?php
    return ob_get_clean();
  }
}
