<?php
/**
 * Shapes Settings
 * Display the shapes settings meta box.
 *
 * @author      Aisconverse
 * @category    Admin
 * @package     Treweler/Admin/Meta Boxes
 * @version     1.10
 */

namespace TrewelerPro;

use TWER_Meta_Boxes;

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Meta_Box_Shape_Settings', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Meta_Boxes', false ) ) {
  include_once TWER_ABSPATH . 'includes/admin/meta-boxes/abstract-class-twer-meta-boxes.php';
}

/**
 * TWER_Meta_Box_Shape_Settings Class.
 */
class TWER_Meta_Box_Shape_Settings extends TWER_Meta_Boxes {

  protected function setNestedFields() {
  }
  protected function setNestedTabs() {
  }
  protected function set_tabs() {
  }

  protected function get_maps() {
	  global $wpdb;

	  $post_id = get_the_ID();

	  $setMap = $post_id ? get_post_meta($post_id, '_treweler_map_id', true) : [];

	  $maps_posts = $wpdb->get_results($wpdb->prepare("
        SELECT ID, post_title
        FROM {$wpdb->prefix}posts
        WHERE post_type = %s
        AND post_status = %s
        ORDER BY post_title ASC
    ", 'map', 'publish'));

	  $maps = [];
	  foreach ($maps_posts as $mapEntry) {
		  $maps[$mapEntry->ID] = $mapEntry->post_title;
	  }

	  if (is_array($setMap) && !empty($setMap)) {
		  $maps_sorted = [];
		  foreach ($setMap as $mapKey) {
			  if ('publish' !== get_post_status($mapKey)) {
				  continue;
			  }

			  $maps_sorted[$mapKey] = $maps[$mapKey];
			  unset($maps[$mapKey]);
		  }
		  $maps = $maps_sorted + $maps;
	  }

	  return $maps;
  }

  protected function set_fields() {

    $this->fields = [
      [
        'type'        => 'select',
        'name'        => 'map_id',
        'class'       => [ 'js-map-multiselect' ],
        'atts'        => [ 'multiple' ],
        'group_class' => 'form-group col',
        'label'       => esc_html__( 'Shape Maps', 'treweler' ),
        'options'     => $this->get_maps(),
        'selected'    => ''
      ],
    ];
  }

}
