<?php
/**
 * Templates Details
 * Display the templates settings details meta box.
 *
 * @author      Aisconverse
 * @category    Admin
 * @package     Treweler/Admin/Meta Boxes
 * @version     1.05
 */

namespace TrewelerPro;

use TWER_Meta_Boxes;

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Meta_Box_Templates_Details', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Meta_Boxes', false ) ) {
  include_once TWER_ABSPATH . 'includes/admin/meta-boxes/abstract-class-twer-meta-boxes.php';
}

/**
 * TWER_Meta_Box_Templates_Details Class.
 */
class TWER_Meta_Box_Templates_Details extends TWER_Meta_Boxes {

  protected $metabox = 'twer-templates';

  protected function setNestedTabs() {
    if ( ! empty( $this->get_all_custom_fields() ) ) {
      $this->nestedTabs = [
        'rootTab' => 'custom-fields',
        'tabs'    => [
          'popup-custom-fields'           => esc_html__( 'Popup', 'treweler' ),
          'locator-preview-custom-fields' => esc_html__( 'Locator Preview', 'treweler' ),
          'locator-custom-fields'         => esc_html__( 'Locator', 'treweler' )
        ]
      ];
    }
  }

  protected function setNestedFields() {
    $popupCustomFields = $locatorPreviewCustomFields = $locatorCustomFields = [];
    if ( ! empty( $this->get_all_custom_fields() ) ) {
      $popupCustomFields = $this->tabs_mapping( $this->get_custom_fields(), 'popup-custom-fields' );
      $locatorPreviewCustomFields = $this->tabs_mapping( $this->get_custom_fields( '_locator_preview' ), 'locator-preview-custom-fields' );
      $locatorCustomFields = $this->tabs_mapping( $this->get_custom_fields( '_locator' ), 'locator-custom-fields' );


      $this->nestedFields = array_merge( $popupCustomFields, $locatorPreviewCustomFields, $locatorCustomFields );
    }
  }

  protected function set_tabs() {
    $this->tabs = [
      'general'       => esc_html__( 'General', 'treweler' ),
      'marker-styles' => esc_html__( 'Marker Styles', 'treweler' ),
      'popup-styles'  => esc_html__( 'Popup Styles', 'treweler' ),
    ];

    $this->tabs['custom-fields'] = esc_html__( 'Custom Fields', 'treweler' );
    $this->tabs['text-label'] = esc_html__( 'Text Label', 'treweler' );
  }

  protected function set_fields() {
    $general = $this->tabs_mapping( $this->get_general_fields(), 'general' );
    $styles = $this->tabs_mapping( $this->get_styles_fields(), 'marker-styles' );
    $popup = $this->tabs_mapping( $this->get_popup_fields(), 'popup-styles' );
    $customFields = [];
    if ( ! empty( $this->get_all_custom_fields() ) ) {
      $customFields = $this->tabs_mapping( $this->get_custom_fields1(), 'custom-fields' );
    } else {
      $customFields = $this->tabs_mapping( $this->get_custom_fields_empty(), 'custom-fields' );
    }
    $labels = $this->tabs_mapping( $this->get_label_fields(), 'text-label' );

    $this->fields = array_merge( $general, $styles, $popup, $labels, $customFields );
  }

  public function get_custom_fields_empty() {
    return [
      [
        'type'  => 'message',
        'name'  => 'message',
        'label' => '',
        'style' => 'style="padding: 26px 23px"',
        'value' => esc_html__( 'No custom field created.', 'treweler' )
      ],
    ];
  }

  public function get_all_custom_fields() {
	  global $wpdb;

	  // Create a prepared SQL statement to retrieve custom fields
	  $query = $wpdb->prepare("
        SELECT * 
        FROM {$wpdb->prefix}posts
        WHERE post_status = %s
        AND post_type = %s
        ORDER BY post_date ASC
    ", 'publish', 'twer-custom-fields');

	  return $wpdb->get_results($query);
  }

  public function fill_custom_fields( $includes_custom_fields_posts, $prefix, $meta_data, $hidden = false ) {
    $current_custom_fields = [];
    if ( ! empty( $includes_custom_fields_posts ) ) {
      foreach ( $includes_custom_fields_posts as $field_post ) {
        $field_post_ID = isset( $field_post->ID ) ? $field_post->ID : 0;
        $field_post_title = isset( $field_post->post_title ) ? $field_post->post_title : '';
        $meta = twer_get_data( $field_post_ID );

        $field_post_type = isset( $meta->custom_field_type ) ? $meta->custom_field_type : 'text';
        $field_post_key = isset( $meta->custom_field_key ) ? $meta->custom_field_key : 'field-1488228';
        //$field_post_show = isset($meta->custom_field_show) ? $meta->custom_field_show : '';


        $tr_class = ! $hidden ? 'js-ui-slider-item' : 'js-hidden-item d-none';
        $disabled = ! $hidden ? '' : 'disabled';

        $disableVisibilityField = '';

        if ( $hidden ) {
          $disableVisibilityField = 'disabled';
        }


        switch ( $field_post_type ) {
          case 'separator' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : '';


            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'label'        => $field_post_title,
              'type'         => 'number',
              'append'       => 'px',
              'placeholder'  => '0',
              'id'           => uniqid( 'separator-', true ),
              'group_class'  => [ 'col-130' ],
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'atts'         => [ 'min="0"', 'step="1"', 'disabled' ],
              'name'         => $field_post_key,
              'value'        => $default,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
          case 'html' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : [];
            $html_default = isset( $default['html'] ) ? $default['html'] : '';
            $value = [
              [
                'type'  => 'textarea',
                'name'  => 'html',
                'group_class'  => 'form-group col-auto',
                'id'    => uniqid( 'html-', true ),
                'atts'  => [ 'cols="60"', 'rows="13"', 'disabled' ],
                'value' => $html_default
              ],
            ];

            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'group',
              'labelThClass' => 'flex-column align-items-baseline',
              'name'         => $field_post_key,
              'label'        => $field_post_title,
              'group'        => $value,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];

            break;
          case 'multiselect' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : [];
            $multiselect_default = isset( $default['multiselect'] ) ? $default['multiselect'] : [];
            $multiselect_default = explode( "\n", str_replace( "\r", "", $multiselect_default ) );

            $selectedIds = [];
            $verifiedSelected = [];
            if ( is_array( $multiselect_default ) && ! empty( $multiselect_default ) ) {
              foreach ( $multiselect_default as $idSelectValue => &$selectValue ) {
                if ( empty( $selectValue ) ) {
                  continue;
                }

                if ( $selectValue[0] === '"' && $selectValue[ strlen( $selectValue ) - 1 ] === '"' ) {
                  $StrRepFirst = substr( $selectValue, 1 );
                  $StrRepLast = substr( $StrRepFirst, 0, - 1 );
                  $selectValue = $StrRepLast;
                  $selectedIds[] = $idSelectValue;
                }

                $verifiedSelected[ $idSelectValue ] = $selectValue;
              }
              unset( $selectValue );
            }


            $value = [
              [
                'type'        => 'select',
                'name'        => 'multiselect',
                'id'          => uniqid( 'multiselect-', true ),
                'class'       => [ 'js-field-multiselect' ],
                'atts'        => [ 'multiple', 'disabled' ],
                'group_class' => 'form-group col-441',
                'options'     => $verifiedSelected,
                'selected'    => $selectedIds
              ],
            ];
            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'group',
              'name'         => $field_post_key,
              'label'        => $field_post_title,
              'group'        => $value,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
          case 'line' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : [];
            $margin_top_default = isset( $default['margin_top'] ) ? $default['margin_top'] : '';
            $margin_bottom_default = isset( $default['margin_bottom'] ) ? $default['margin_bottom'] : '';
            $value = [
              [
                'type'         => 'number',
                'name'         => 'margin_top',
                'placeholder'  => '14',
                'append'       => 'px',
                'id'           => uniqid( 'line-margin-top', true ),
                'label_inline' => esc_html__( 'Top', 'treweler' ),
                'atts'         => [ 'step="1"', 'disabled' ],
                'value'        => $margin_top_default
              ],
              [
                'type'         => 'number',
                'name'         => 'margin_bottom',
                'placeholder'  => '14',
                'append'       => 'px',
                'id'           => uniqid( 'line-margin-bottom', true ),
                'label_inline' => esc_html__( 'Bottom', 'treweler' ),
                'atts'         => [ 'step="1"', 'disabled' ],
                'value'        => $margin_bottom_default
              ],
            ];

            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'group',
              'name'         => $field_post_key,
              'label'        => $field_post_title,
              'group'        => $value,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];

            break;
          case 'number' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : [];
            $number_default = isset( $default['number'] ) ? $default['number'] : '';
            $value = [
              [
                'type'  => 'text',
                'name'  => 'number',
                'id'    => uniqid( 'number-', true ),
                'atts'  => [ 'step="any"', 'disabled' ],
                'value' => $number_default
              ],
            ];

            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'group',
              'name'         => $field_post_key,
              'label'        => $field_post_title,
              'group'        => $value,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];

            break;
          case 'category' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : '';
            if ( is_array( $default ) ) {
              $default = isset( $default['style'] ) ? $default['style'] : '';
            }
            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'label'        => $field_post_title,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'textarea',
              'labelThClass' => '',
              'group_class'  => 'form-group col-200',
              'id'    => uniqid( 'category-', true ),
              'atts'         => [ 'cols="60"', 'rows="4"', 'disabled' ],
              'name'         => $field_post_key,
              'value'        => $default,
              'class'        => 'hide-field',
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
          case 'text' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : '';
            $showAsMarkerTitle = false;
            if ( is_array( $default ) ) {

              $showAsMarkerTitle = isset( $default['showTitle'] ) ? $default['showTitle'] : '';
              $showAsMarkerTitle = is_array( $showAsMarkerTitle ) ? reset( $showAsMarkerTitle ) : $showAsMarkerTitle;

              $default = isset( $default['text'] ) ? $default['text'] : '';


            }


            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'label'        => $field_post_title,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'textarea',
              'labelThClass' => $showAsMarkerTitle ? '' : 'flex-column align-items-start',
              'group_class'  => $showAsMarkerTitle ? 'form-group col-200' : 'form-group col-auto',
              'id'    => uniqid( 'text-', true ),
              'atts'         => [ 'cols="60"', 'rows="4"', 'disabled' ],
              'name'         => $field_post_key,
              'value'        => $default,
              'class'        => $showAsMarkerTitle ? 'hide-field' : 'form-control form-control-sm',
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
          case 'true_false' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : [];
            $true_false_default = isset( $default['true_false'] ) ? $default['true_false'] : [];
            $true_false_default = is_array( $true_false_default ) ? reset( $true_false_default ) : $true_false_default;


            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'group',
              'name'         => $field_post_key,
              'label'        => $field_post_title,
              'group'        => [
                [
                  'type'    => 'checkbox',
                  'name'    => 'target',
                  'id'    => uniqid( 'true-false-', true ),
                  'atts'    => [ 'disabled' ],
                  'checked' => $true_false_default
                ]
              ],
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];

            break;
          case 'email' :
          case 'link' :
          case 'phone' :
          case 'button' :

            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : [];

            $text_placeholder = esc_html__( 'URL', 'treweler' );
            $stype = '';
            $fieldType = '';
            $urlKey = '';
            $targetKey = 'target';
            if ( $field_post_type === 'email' ) {
              $stype = $fieldType = $urlKey = 'email';
              $text_placeholder = esc_html__( 'Email', 'treweler' );
            } elseif ( $field_post_type === 'phone' ) {
              $stype = $fieldType = $urlKey = 'tel';
              $text_placeholder = esc_html__( 'Phone Number', 'treweler' );
            } elseif ( $field_post_type === 'link' ) {
              $stype = $fieldType = $urlKey = 'url';
            } elseif ( $field_post_type === 'button' ) {
              $stype = 'btn';
              $urlKey = 'btn_url';
              $targetKey = 'btn_target';
              $fieldType = 'url';
            }


            $key_default = $stype . '_label';


            $text_default = isset( $default[ $key_default ] ) ? $default[ $key_default ] : '';
            $url_default = isset( $default[ $urlKey ] ) ? $default[ $urlKey ] : '';
            $target_default = isset( $default[ $targetKey ] ) ? $default[ $targetKey ] : [];
            $target_default = is_array( $target_default ) ? reset( $target_default ) : $target_default;

            $value = [
              [
                'type'        => 'text',
                'name'        => 'text',
                'placeholder' => esc_html__( 'Text', 'treweler' ),
                'atts'        => [ 'disabled' ],
                'id'    => uniqid( 'group-', true ),
                'value'       => $text_default
              ],
              [
                'type'        => $fieldType,
                'name'        => 'url',
                'placeholder' => $text_placeholder,
                'atts'        => [ 'disabled' ],
                'id'    => uniqid( 'group-', true ),
                'value'       => $url_default
              ],
            ];


            if ( $field_post_type === 'link' ) {
              $value[] = [
                'type'         => 'checkbox',
                'name'         => 'target',
                'atts'         => [ 'disabled' ],
                'id'    => uniqid( 'group-', true ),
                'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
                'checked'      => $target_default
              ];
            } elseif ( $field_post_type === 'button' ) {
              $value[] = [
                'type'         => 'checkbox',
                'name'         => 'btn_target',
                'id'    => uniqid( 'group-', true ),
                'atts'         => [ 'disabled' ],
                'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
                'checked'      => $target_default
              ];
            }

            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'group',
              'name'         => $field_post_key,
              'label'        => $field_post_title,
              'group'        => $value,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
        }
      }
    }

    return $current_custom_fields;
  }

  public function get_custom_fields1() {
    return [
      [
        'type'        => 'checkbox',
        'name'        => 'popup_show',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Show popup', 'treweler' ),
        'checked'     => false
      ],
    ];
  }

  public function get_custom_fields( $customFieldsListNamePostfix = '' ) {
    $meta_data = twer_get_data();
    $prefix = '_treweler_';


    // Get saved custom fields
    $customFieldsListName = 'custom_fields_list' . $customFieldsListNamePostfix;
    $current_custom_fields_ids = isset( $meta_data->$customFieldsListName ) ? $meta_data->$customFieldsListName : '';

    // Get all includes/saved custom fields and save it to array
    $includes_custom_fields_posts = ! empty( $current_custom_fields_ids ) ? get_posts( [
      'post_status' => 'publish',
      'post_type'   => 'twer-custom-fields',
      'orderby'     => 'post__in',
      'include'     => $current_custom_fields_ids
    ] ) : [];

    $fields_custom = array_merge(
      [

        [
          'row_class'   => 'twer-tr-toggle twer-row js-twer-row',
          'type'        => 'checkbox',
          'name'        => 'custom_field_show' . $customFieldsListNamePostfix,
          'group_class' => 'form-group col-auto',
          'id'    => uniqid( 'show-custom-fields-' ),
          'label'       => esc_html__( 'Show Custom Fields', 'treweler' ),
          'checked'     => false
        ],
        [
          'row_class' => 'twer-tr-toggle twer-row js-twer-row d-none',
          'label'     => esc_html__( 'Custom Fields', 'treweler' ),
          'type'      => 'text',
          'atts'      => [ 'hidden' ],
          'id'    => uniqid( 'custom-fields-list-' ),
          'class'     => [ 'js-twer-custom-fields-list' ],
          'name'      => 'custom_fields_list' . $customFieldsListNamePostfix,
          'value'     => '',
        ],
      ],
      $this->fill_custom_fields( $includes_custom_fields_posts, $prefix, $meta_data ),
      $this->fill_custom_fields( $this->get_all_custom_fields(), $prefix, $meta_data, true )
    );


    $select_fields = [
      'none' => __( 'Select a Field', 'treweler' ),
    ];
    // Get all excluded/not saved custom fields and save it to array
    $excluded_custom_fields_posts = get_posts( [
      'post_status'    => 'publish',
      'post_type'      => 'twer-custom-fields',
      'posts_per_page' => - 1,
      'numberposts'    => - 1,
      'exclude'        => $current_custom_fields_ids
    ] );

    foreach ( $excluded_custom_fields_posts as $excluded_custom_field_post ) {
      $id = isset( $excluded_custom_field_post->ID ) ? $excluded_custom_field_post->ID : 0;
      $label = isset( $excluded_custom_field_post->post_title ) ? $excluded_custom_field_post->post_title : '';
      /*$meta = twer_get_data( $id );
      $field_type = isset( $meta->custom_field_type ) ? $meta->custom_field_type : '';*/


      if ( empty( $label ) ) {
        $label = sprintf( __( 'Field: %d' ), $id );
      }
      if ( $id ) {
        $select_fields[ $id ] = $label;
      }
    }

    $fields_custom[] = [
      'type'  => 'group',
      'name'  => 'custom_fields',
      'label' => esc_html__( 'Add fields', 'treweler' ),
      'group' => [
        [
          'type'        => 'select',
          'name'        => 'list',
          'group_class' => [ 'col-200' ],
          'id'    => uniqid( 'all-custom-fields-' ),
          'class'       => [ 'js-custom-fields-list' ],
          'options'     => $select_fields,
          'selected'    => 'none',
        ],
        [
          'type'         => 'button',
          'name'         => 'add',
          'id'    => uniqid( 'add-new-custom-field-' ),
          'group_class'  => 'form-group col-auto',
          'class'        => [ 'js-add-custom-field' ],
          'label_inline' => __( 'Add Field', 'treweler' )
        ],
      ]
    ];

    return $fields_custom;
  }


  public function getLocatorFields() {
    return [
      [
        'type'        => 'checkbox',
        'name'        => 'some_field',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Some Control 2', 'treweler' ),
        'checked'     => false
      ],
    ];
  }

  public function get_styles_fields() {
    return [
      [
        'type'      => 'select',
        'name'      => 'marker_style',
        'id'        => 'marker_style',
        'row_class' => 'twer-row js-twer-row twer-tr-toggle',
        'label'     => esc_html__( 'Style', 'treweler' ),
        'options'   => [
          'default'          => esc_html__( 'Point', 'treweler' ),
          'dot-default'      => esc_html__( 'Dot', 'treweler' ),
          'balloon-default'  => esc_html__( 'Balloon', 'treweler' ),
          'triangle-default' => esc_html__( 'Triangle', 'treweler' ),
          'custom'           => esc_html__( 'Custom Marker', 'treweler' )
        ],
        'selected'  => 'default'
      ],

      [
        'type'        => 'colorpicker',
        'name'        => 'point_color',
        'label'       => esc_html__( 'Marker Color', 'treweler' ),
        'value'       => '#4b7715',
        'id'          => 'markerColor',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'default' ], 'show' => true ]
      ],
      [
        'type'        => 'colorpicker',
        'name'        => 'point_halo_color',
        'label'       => esc_html__( 'Halo Color', 'treweler' ),
        'value'       => '#ffffff',
        'id'          => 'marker_halo_color',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'default' ], 'show' => true ]
      ],
      [
        'type'        => 'number',
        'name'        => 'point_halo_opacity',
        'group_class' => 'form-group d-flex align-items-center flex-nowrap col col-242',
        'atts'        => [ 'min="0"', 'step="0.1"', 'max="1"' ],
        'placeholder' => '0.5',
        'id'          => 'treweler-marker-halo-opacity',
        'label'       => esc_html__( 'Halo Opacity', 'treweler' ),
        'value'       => '',
        'help'        => esc_html__( 'The opacity property can take a value from 0 - 1. The lower value, the more transparent.',
          'treweler' ),
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'default' ], 'show' => true ]
      ],

      [
        'type'        => 'checkbox',
        'name'        => 'dot_icon_show',
        'label'       => esc_html__( 'Show Icon', 'treweler' ),
        'checked'     => false,
        'id'          => 'treweler-picker-dot',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],
      [
        'type'        => 'iconpicker',
        'name'        => 'dot_icon_picker',
        'id'          => 'treweler-dot-icon-picker',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Select Icon', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'treweler-picker-dot', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot_icon',
        'label'      => esc_html__( 'Icon', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'id'          => 'dot_icon_color',
            'group_class' => 'form-group col-auto',
            'value'       => '#ffffff',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '15',
            'id'          => 'treweler-dot-icon-size',
            'group_class' => [ 'col-130' ],
            'append'      => 'px',
            'value'       => ''
          ],
        ],
        'toggle-row' => [ 'related' => 'treweler-picker-dot', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot',
        'label'      => esc_html__( 'Marker', 'treweler' ),
        'group'      => [

          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#4b7715',
            'id'          => 'marker_dotcenter_color',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '12',
            'append'      => 'px',
            'id'          => 'treweler-marker-size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'group_class' => [ 'col-130' ],
            'value'       => '',

          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot_border',
        'label'      => esc_html__( 'Border', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'id'          => 'marker_border_color',
            'label'       => esc_html__( 'Border Color', 'treweler' ),
            'value'       => '#ffffff',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '0',
            'append'      => 'px',
            'id'          => 'treweler-marker-border-width',
            'group_class' => [ 'col-130' ],
            'atts'        => [ 'min="0"', 'step="1"' ],
            'value'       => '',
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot_corner_radius',
        'label'      => esc_html__( 'Corner Radius', 'treweler' ),
        'group'      => [
          [
            'type'        => 'number',
            'name'        => 'size',
            'id'          => 'treweler-marker-corner-radius-group',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '50',
            'value'       => '',
          ],
          [
            'type'        => 'select',
            'name'        => 'units',
            'id'          => 'treweler-marker-corner-radius-group-select',
            'group_class' => [ 'col-130' ],
            'options'     => [
              '%'  => esc_html__( '%', 'treweler' ),
              'px' => esc_html__( 'px', 'treweler' ),
            ],
            'selected'    => '%'
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],


      [
        'type'        => 'checkbox',
        'name'        => 'balloon_icon_show',
        'label'       => esc_html__( 'Show Icon', 'treweler' ),
        'checked'     => false,
        'id'          => 'treweler-picker',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],
      [
        'type'        => 'iconpicker',
        'name'        => 'balloon_icon_picker',
        'group_class' => 'form-group col-auto',
        'id'          => 'treweler-balloon-icon-picker',
        'label'       => esc_html__( 'Select Icon', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'treweler-picker', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon_icon',
        'label'      => esc_html__( 'Icon', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#ffffff',
            'id'          => 'balloon_icon_color',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '15',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'id'          => 'treweler-balloon-icon-size',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
        ],
        'toggle-row' => [ 'related' => 'treweler-picker', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon',
        'label'      => esc_html__( 'Marker', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'id'          => 'marker_color_balloon',
            'value'       => '#4b7715',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '18',
            'id'          => 'treweler-marker-size-balloon',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'group_class' => [ 'col-130' ],
            'append'      => 'px',
            'value'       => '',
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon_border',
        'label'      => esc_html__( 'Border', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#4b7715',
            'id'          => 'marker_border_color_balloon',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'name'        => 'size',
            'placeholder' => '4',
            'append'      => 'px',
            'id'          => 'treweler-marker-border-width-balloon',
            'group_class' => [ 'col-130' ],
            'value'       => '',
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon_dot',
        'label'      => esc_html__( 'Dot', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#ffffff',
            'id'          => 'marker_dot_color',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '8',
            'id'          => 'treweler-marker-dot-size',
            'append'      => 'px',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],


      [
        'type'        => 'colorpicker',
        'name'        => 'triangle_color',
        'id'          => 'marker_color_triangle',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Marker Color', 'treweler' ),
        'value'       => '#4b7715',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'triangle-default' ] ]
      ],
      [
        'type'        => 'number',
        'name'        => 'triangle_width',
        'placeholder' => '12',
        'id'          => 'treweler-marker-width-triangle',
        'append'      => 'px',
        'label'       => esc_html__( 'Marker Width', 'treweler' ),
        'atts'        => [ 'min="0"', 'step="1"' ],
        'value'       => '',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'triangle-default' ] ]
      ],
      [
        'type'        => 'number',
        'name'        => 'triangle_height',
        'placeholder' => '10',
        'append'      => 'px',
        'id'          => 'treweler-marker-height-triangle',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'label'       => esc_html__( 'Marker Height', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'triangle-default' ] ]
      ],


      [
        'type'       => 'image',
        'name'       => 'custom_marker_img',
        'id'         => 'thumbnail_id',
        'label'      => esc_html__( 'Custom Marker', 'treweler' ),
        'value'      => '',
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ]
      ],
      [
        'type'       => 'select',
        'name'       => 'custom_marker_position',
        'label'      => esc_html__( 'Position', 'treweler' ),
        'options'    => $this->position_options,
        'id'         => 'marker_position',
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ],
        'selected'   => 'center'
      ],
      [
        'type'        => 'number',
        'name'        => 'custom_marker_size',
        'placeholder' => '42',
        'append'      => 'px',
        'id'          => 'treweler-marker-img-size',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'label'       => esc_html__( 'Marker Size', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ]
      ],
      [
        'type'       => 'select',
        'name'       => 'custom_marker_cursor',
        'id'         => 'marker_cursor',
        'label'      => esc_html__( 'Cursor on Hover', 'treweler' ),
        'options'    => [
          'pointer' => esc_html__( 'Pointer', 'treweler' ),
          'grab'    => esc_html__( 'Grab', 'treweler' ),
        ],
        'selected'   => 'pointer',
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ]
      ],

    ];
  }

  public function get_popup_fields() {
    $oldMetaFields1 = [
      [
        'type'  => 'group',
        'name'  => 'popup_heading',
        'label' => esc_html__( 'Heading', 'treweler' ),
        'group' => [
          [
            'type'        => 'text',
            'group_class' => [ 'col-260' ],
            'name'        => 'text',
            'value'       => '',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'placeholder' => '16',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-200' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_subheading',
        'label' => esc_html__( 'Subheading', 'treweler' ),
        'group' => [
          [
            'type'        => 'text',
            'group_class' => [ 'col-260' ],
            'name'        => 'text',
            'value'       => '',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'placeholder' => '13',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-200' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_description',
        'label' => esc_html__( 'Description', 'treweler' ),
        'group' => [
          [
            'type'        => 'textarea',
            'group_class' => [ 'col-260' ],
            'name'        => 'text',
            'atts'        => [ 'cols="30"', 'rows="6" ' ],
            'value'       => '',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'placeholder' => '13',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-200' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
    ];
    $oldMetaFields2 = [
      [
        'type'  => 'group',
        'name'  => 'popup_button',
        'label' => esc_html__( 'Button', 'treweler' ),
        'group' => [
          [
            'type'         => 'text',
            'name'         => 'text',
            'label_inline' => esc_html__( 'Text', 'treweler' ),
            'value'        => '',
          ],
          [
            'type'         => 'url',
            'name'         => 'url',
            'label_inline' => esc_html__( 'URL', 'treweler' ),
            'value'        => '',
          ],
          [
            'type'         => 'colorpicker',
            'name'         => 'color',
            'group_class'  => 'form-group col-auto d-flex flex-column',
            'label_inline' => esc_html__( 'Color', 'treweler' ),
            'value'        => '#4a7805'
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'target',
            'group_class'  => 'form-group col-auto',
            'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
            'class'        => [ 'twer-switcher--vertical' ],
            'checked'      => false
          ],
        ]
      ],
      [
        'type'     => 'select',
        'name'     => 'popup_content_align',
        'label'    => esc_html__( 'Content align', 'treweler' ),
        'options'  => [
          'left'   => esc_html__( 'Left', 'treweler' ),
          'center' => esc_html__( 'Center', 'treweler' ),
        ],
        'selected' => 'left'
      ],
    ];

    $metaFields1 = [
      [
        'type'        => 'checkbox',
        'name'        => 'popup_show',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Show popup', 'treweler' ),
        'checked'     => false
      ],
    ];

    $metaFields2 = [
      [
        'type'        => 'gallery',
        'name'        => 'popup_gallery',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Images', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'popup_gallery_show',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Enable Image Gallery', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'     => 'select',
        'name'     => 'popup_gallery_position',
        'label'    => esc_html__( 'Position', 'treweler' ),
        'id'       => 'treweler-popup-image-position',
        'options'  => [
          'right' => esc_html__( 'Right', 'treweler' ),
          'top'   => esc_html__( 'Top', 'treweler' ),
        ],
        'selected' => 'right'
      ],
      [
        'type'        => 'number',
        'name'        => 'popup_gallery_width',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '160',
        'id'          => 'treweler-popup-image-width',
        'label'       => esc_html__( 'Image width', 'treweler' ),

        'value' => ''
      ],
    ];
    $metaFields3 = [
      [
        'type'  => 'group',
        'name'  => 'popup_size',
        'label' => esc_html__( 'Min width and height', 'treweler' ),
        'group' => [
          [
            'type'         => 'number',
            'name'         => 'height',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'label_inline' => esc_html__( 'Height', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'width',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'label_inline' => esc_html__( 'Width', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],
      [
        'type'        => 'number',
        'name'        => 'popup_border_radius',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '0',
        'label'       => esc_html__( 'Corner Radius', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'     => 'select',
        'name'     => 'popup_style',
        'label'    => esc_html__( 'Style', 'treweler' ),
        'options'  => [
          'light' => esc_html__( 'Light', 'treweler' ),
          'dark'  => esc_html__( 'Dark', 'treweler' ),
        ],
        'selected' => 'light'
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_open_on',
        'label' => esc_html__( 'Open popup on', 'treweler' ),
        'group' => [
          [
            'type'         => 'select',
            'name'         => 'action',
            'id'           => 'treweler-popup-open-group-open-on',
            'label_inline' => esc_html__( 'Open popup on', 'treweler' ),
            'options'      => [
              'hover'       => esc_html__( 'Hover', 'treweler' ),
              'click'       => esc_html__( 'Click', 'treweler' ),
              'always_open' => esc_html__( 'Always open', 'treweler' ),
            ],
            'selected'     => 'hover'
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'default',
            'group_class'  => 'form-group col-auto',
            'id'           => 'treweler-popup-open-group-open-default',
            'label_inline' => esc_html__( 'Open by default', 'treweler' ),
            'class'        => [ 'twer-switcher--vertical' ],
            'checked'      => false
          ],
        ]
      ],

      [
        'type'  => 'group',
        'name'  => 'popup_close_button',
        'label' => esc_html__( 'Show Close Icon', 'treweler' ),
        'group' => [
          [
            'type'         => 'checkbox',
            'name'         => 'show',
            'group_class'  => 'form-group col-auto',
            'checked'      => false
          ],
          [
            'type'         => 'select',
            'name'         => 'style',
            'group_class'  => ['col-153'],
            'options'      => [
              'dark'       => esc_html__( 'Dark', 'treweler' ),
              'light'       => esc_html__( 'Light', 'treweler' ),
            ],
            'selected'     => 'dark'
          ],
        ]
      ],
    ];

    if(TWER_OLD_POPUP_FIELDS) {
      $metaFields = array_merge($metaFields1, $oldMetaFields1,$metaFields2,$oldMetaFields2, $metaFields3);
    } else {
      $metaFields = array_merge($metaFields1,$metaFields2, $metaFields3);
    }

    return $metaFields;
  }

  public function get_general_fields() {
    return [
      [
        'type'        => 'checkbox',
        'name'        => 'marker_enable_clustering',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Enable Clustering', 'treweler' ),
        'checked'     => true
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'marker_enable_center_on_click',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Center On Click', 'treweler' ),
        'checked'     => true
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'marker_hide_in_locator',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Hide in Locator', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'marker_ignore_filters',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Ignore Filters', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'marker_ignore_radius_filter',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Ignore Radius Filter', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'  => 'group',
        'name'  => 'marker_click_offset',
        'label' => esc_html__( 'Marker Offset', 'treweler' ),
        'group' => [
          [
            'type'         => 'number',
            'name'         => 'top',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '0',
            'label_inline' => esc_html__( 'Top', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'bottom',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '0',
            'label_inline' => esc_html__( 'Bottom', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'left',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '0',
            'label_inline' => esc_html__( 'Left', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'right',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '0',
            'label_inline' => esc_html__( 'Right', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'marker_open_link_by_click',
        'label'       => esc_html__( 'Open Link On Click', 'treweler' ),
        'checked'     => false,
        'row_class'   => 'twer-tr-toggle twer-row js-twer-row',
        'group_class' => 'form-group col-auto',
      ],
      [
        'type'   => 'group',
        'name'  => 'marker_link',
        'label'  => esc_html__( 'Link', 'treweler' ),
        'group'  => [
          [
            'type'        => 'url',
            'name'        => 'url',
            'placeholder' => esc_html__( 'URL', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'target',
            'group_class'  => 'form-group col-auto',
            'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
            'checked'      => false
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_open_link_by_click', 'value' => true ]
      ],
    ];
  }

  public function get_label_fields() {
    return [
      [
        'type'        => 'checkbox',
        'name'        => 'label_enable',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Show Text Label', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'        => 'textarea',
        'group_class' => [ 'col-445' ],
        'name'        => 'label_description',
        'atts'        => [ 'cols="30"', 'rows="6" ' ],
        'label'       => esc_html__( 'Label Text', 'treweler' ),
        'value'       => ''
      ],
      [
        'row_class'   => 'twer-tr-toggle twer-row js-twer-row',
        'type'        => 'checkbox',
        'name'        => 'label_has_bg',
        'group_class' => 'form-group col-auto',
        'id'          => 'treweler-label-has-bg',
        'label'       => esc_html__( 'Show Background', 'treweler' ),
        'checked'     => true,
      ],
      [
        'type'        => 'number',
        'name'        => 'label_border_radius',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '0',
        'label'       => esc_html__( 'Corner Radius', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'treweler-label-has-bg', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'label_padding',
        'label'      => esc_html__( 'Text Padding', 'treweler' ),
        'toggle-row' => [ 'related' => 'treweler-label-has-bg', 'value' => true ],
        'group'      => [
          [
            'type'         => 'number',
            'name'         => 'top',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '8',
            'label_inline' => esc_html__( 'Top', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'bottom',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '6',
            'label_inline' => esc_html__( 'Bottom', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'left',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '17',
            'label_inline' => esc_html__( 'Left', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'right',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class'  => [ 'col-130' ],
            'placeholder'  => '17',
            'label_inline' => esc_html__( 'Right', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],
      [
        'type'     => 'select',
        'name'     => 'label_position',
        'label'    => esc_html__( 'Position', 'treweler' ),
        'options'  => $this->position_options,
        'selected' => 'left'
      ],
      [
        'type'        => 'colorpicker',
        'name'        => 'label_color',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Font Color', 'treweler' ),
        'value'       => '#000000'
      ],
      [
        'type'        => 'number',
        'name'        => 'label_size',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '12',
        'label'       => esc_html__( 'Font Size', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'     => 'select',
        'name'     => 'label_weight',
        'label'    => esc_html__( 'Font Weight', 'treweler' ),
        'options'  => $this->font_weight_options,
        'selected' => '400'
      ],
      [
        'type'        => 'number',
        'name'        => 'label_letter_spacing',
        'atts'        => [ 'min="0"', 'step="0.1"' ],
        'append'      => 'px',
        'placeholder' => '0',
        'label'       => esc_html__( 'Letter Spacing', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'number',
        'name'        => 'label_line_height',
        'atts'        => [ 'min="0"', 'step="0.1"' ],
        'placeholder' => '1.3',
        'label'       => esc_html__( 'Line Height', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'number',
        'name'        => 'label_margin',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '5',
        'label'       => esc_html__( 'Margin', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'label_marker_hide',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Hide Marker', 'treweler' ),
        'checked'     => false
      ],
    ];
  }


  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct();

    add_filter( $this->metabox . '_assign_fields', [ $this, 'sort_tab_fields' ], 10, 2 );
    add_filter( $this->metabox . '_assign_nestedFields', [ $this, 'sort_tab_fields' ], 11, 2 );
  }


  public function sort_tab_fields( $fields, $metabox ) {
    $sorted_fields = [];

    foreach ( $fields as $field ) {
      $sorted_fields[ $field['tab'] ][] = $field;
    }

    return $sorted_fields;
  }

}
