<?php
/**
 * Treweler Admin Functions
 *
 * @package  Treweler/Admin/Functions
 * @version  1.14
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


function twerUpdateMetaForMarkerWithTemplate( $markerID, $templateId, $ignoreTemplateMeta = []) {
	$marker_meta_data = twer_get_data( $markerID );
	$template_data    = twerTemplateMetaDiff( twer_get_data( $templateId ) );
	$marker_meta_data = twerTemplateMerge( $marker_meta_data, $template_data );

	if ( ! empty( $marker_meta_data ) ) {
		foreach ( $marker_meta_data as $metaKey => $metaValue ) {
			if(in_array($metaKey, $ignoreTemplateMeta, true)) continue;

			switch ( $metaKey ) {
				case 'marker_enable_clustering' :
				case 'marker_enable_center_on_click' :
				case 'marker_hide_in_locator' :
				case 'marker_ignore_filters' :
				case 'marker_style' :
				case 'marker_ignore_radius_filter' :
				case 'marker_click_offset' :
				case 'picker_dot' :
				case 'dot_icon_picker' :
				case 'dot_icon_color' :
				case 'dot_icon_size' :
				case 'marker_dotcenter_color' :
				case 'marker_color_balloon' :
				case 'marker_color' :
				case 'marker_halo_color' :
				case 'marker_halo_opacity' :
				case 'marker_size_balloon' :
				case 'marker_border_color_balloon' :
				case 'marker_border_width_balloon' :
				case 'marker_size' :
				case 'marker_dot_color' :
				case 'marker_dot_size' :
				case 'marker_border_color' :
				case 'marker_border_width' :
				case 'marker_corner_radius' :
				case 'marker_color_triangle' :
				case 'marker_width_triangle' :
				case 'marker_height_triangle' :
				case 'marker_corner_radius_units' :
				case 'popup_show' :
				case 'popup_gallery_show' :
				case 'popup_image_position' :
				case 'popup_image' :
				case 'popup_image_width' :
				case 'popup_content_align' :
				case 'popup_size' :
				case 'popup_border_radius' :
				case 'popup_style' :
				case 'popup_open_group' :
				case 'marker_enable_labels' :
				case 'label_description' :
				case 'label_has_bg' :
				case 'label_border_radius' :
				case 'label_padding' :
				case 'label_position' :
				case 'label_font_color' :
				case 'label_font_size' :
				case 'label_font_weight' :
				case 'label_letter_spacing' :
				case 'label_line_height' :
				case 'label_margin' :
				case 'marker_hide' :

					update_post_meta( $markerID, '_treweler_' . $metaKey, $metaValue );
					update_post_meta( $markerID, '_treweler_' . $metaKey . '_lock', 'close' );
					break;
			}
		}
	}

	update_post_meta( $markerID, '_treweler_label_padding_top_lock', 'close' );
	update_post_meta( $markerID, '_treweler_label_padding_bottom_lock', 'close' );
	update_post_meta( $markerID, '_treweler_label_padding_left_lock', 'close' );
	update_post_meta( $markerID, '_treweler_label_padding_right_lock', 'close' );
	update_post_meta( $markerID, '_treweler_popup_open_group_open_on_lock', 'close' );
	update_post_meta( $markerID, '_treweler_popup_open_group_open_default_lock', 'close' );

	update_post_meta( $markerID, '_treweler_popup_size_height_lock', 'close' );
	update_post_meta( $markerID, '_treweler_popup_size_width_lock', 'close' );


	update_post_meta( $markerID, 'marker_click_offset_top_lock', 'close' );
	update_post_meta( $markerID, 'marker_click_offset_bottom_lock', 'close' );
	update_post_meta( $markerID, 'marker_click_offset_left_lock', 'close' );
	update_post_meta( $markerID, 'marker_click_offset_right_lock', 'close' );
}

function twerIsValidGoogleJSONCredentialsFile(): bool {
	$options = get_option( 'treweler' );

	$isValid       = true;
	$attachment_id = isset( $options['google_sheets_api_json'] ) ? esc_attr( $options['google_sheets_api_json'] ) : '';

	if ( empty( $attachment_id ) ) {
		$isValid = false;
	}

	$file_path = get_attached_file( $attachment_id );

	if ( $file_path && file_exists($file_path) ) {
		$json_string = file_get_contents( $file_path );
		$data        = json_decode( $json_string );

		if ( json_last_error() === JSON_ERROR_NONE ) {
			if ( ! isset( $data->type, $data->private_key_id, $data->private_key, $data->client_email, $data->client_id ) ) {
				$isValid = false;
			}

			$client = new \Google_Client();
			try {
				$client->setAuthConfig( $file_path );
			}catch ( \Google_Service_Exception $e ) {
				$isValid = false;
			}
		} else {
			$isValid = false;
		}
	} else {
		$isValid = false;
	}

	return $isValid;
}

function twerGetGoogleClient(): array {
	try {
		$client                 = new Google\Client();
		$google_sheets_api_json = twerGetOption( 'google_sheets_api_json' );
		if ( empty( $google_sheets_api_json ) ) {
			$error = wp_json_encode( [
				'success' => false,
				'error'   => [
					'code'    => 228,
					'message' => __( 'Google API JSON must not be empty. Please upload a valid API JSON file.', 'treweler' )
				]
			] );
			throw new \Google\Service\Exception( $error );
		}
		$file_path = get_attached_file( $google_sheets_api_json );

		$client->setApplicationName( __( 'Import Markers', 'treweler' ) );
		$client->setScopes( [ Google\Service\Sheets::SPREADSHEETS ] );
		$client->setAccessType( 'offline' );
		$client->setAuthConfig( $file_path );

		return [ 'success' => true, 'data' => $client ];
	}catch ( \Google\Service\Exception $e ) {
		try {
			$message = json_decode( $e->getMessage(), false, 512, JSON_THROW_ON_ERROR );

			return (array) $message;
		}catch ( \JsonException $e ) {
			return [
				'success' => false,
				'error'   => [
					'code'    => 1488,
					'message' => __( 'Problems connecting to the API.', 'treweler' )
				]
			];
		}
	}
}

function twerGetImportedMarkersByKeys( array $markersKeys ): array {
	global $wpdb;
	$keys            = array_map( 'esc_sql', $markersKeys );
	$keys_str        = implode( "','", $keys );
	$sql             = $wpdb->prepare(
		"SELECT p.ID, pm.meta_value AS MarkerKey
    FROM {$wpdb->prefix}posts AS p
    INNER JOIN {$wpdb->prefix}postmeta AS pm ON p.ID = pm.post_id
    WHERE p.post_type = %s AND p.post_status = 'publish' AND pm.meta_key = '_treweler_google_import_marker_key' AND pm.meta_value IN ('{$keys_str}')",
		'marker'
	);
	$results         = $wpdb->get_results( $sql, ARRAY_A );
	$posts_with_keys = [];
	if ( $results ) {
		foreach ( $results as $result ) {
			$posts_with_keys[ $result['ID'] ] = $result['MarkerKey'];
		}
	}

	return $posts_with_keys;
}


/**
 * Fetches the markers for deletion based on the specified import rule.
 *
 * @param string $rule         The rule specifying how to retrieve the markers.
 * @param array  $markersKeys  An array of keys identifying markers.
 * @param int|null $importID The ID related to the import process.
 *
 * @return array An array of post IDs that match the criteria for deletion.
 */
function twerGetMarkersForDeletionByImportRule( string $rule, array $markersKeys, int $importID = null ): array {
	global $wpdb;
	if ($importID === null && isset($_POST['post_ID'])) {
		$importID = (int) $_POST['post_ID'];
	}

	$posts_not_in_keys = [];
	$markersKeys  = array_map( 'esc_sql', $markersKeys );
	$placeholders = implode( ", ", array_fill( 0, count( $markersKeys ), '%s' ) );

	switch ( $rule ) {
		case 'rm_markers_missing_table_early':
			$sql = $wpdb->prepare(
				"SELECT p.ID 
    FROM {$wpdb->prefix}posts AS p
    LEFT JOIN {$wpdb->prefix}postmeta AS pm ON p.ID = pm.post_id AND pm.meta_key = '_treweler_google_import_marker_key'
    WHERE p.post_type = %s 
    AND p.post_status = 'publish'
    AND (pm.meta_value IS NULL OR pm.meta_value NOT IN ($placeholders))",
				array_merge( array( 'marker' ), $markersKeys )
			);

			$posts_not_in_keys = $wpdb->get_col( $sql );

			break;
		case 'rm_markers_missing_table' :

			$sql = $wpdb->prepare(
				"SELECT p.ID 
    FROM {$wpdb->prefix}posts AS p
    INNER JOIN {$wpdb->prefix}postmeta AS pm ON p.ID = pm.post_id
    INNER JOIN {$wpdb->prefix}postmeta AS pm2 ON p.ID = pm2.post_id
    WHERE p.post_type = %s
    AND p.post_status = 'publish'
    AND pm.meta_key = '_treweler_google_import_marker_key'
    AND pm2.meta_key = '_treweler_google_import_id'
    AND pm2.meta_value = %s
    AND pm.meta_value NOT IN ($placeholders)
    AND pm.meta_value != ''",
				array_merge( array( 'marker', $importID ), $markersKeys )
			);

			$posts_not_in_keys = $wpdb->get_col( $sql );
			break;
	}

	return $posts_not_in_keys;
}


/**
 * Retrieves the title of the sheet by its ID.
 *
 * @param int    $sheetId        The ID of the sheet to retrieve the title for.
 * @param string $spreadsheetId  The ID of the spreadsheet containing the sheet.
 *
 * @return string Returns the title of the sheet if found, otherwise returns an empty string.
 */
function twerGetSheetTitleById( int $sheetId, string $spreadsheetId ): string {
	$sheets = twerGetSheetIds($spreadsheetId);
	$sheetTitle = '';

	$sheetsData = $sheets['data']->sheets ?? [];

	if (!empty($sheetsData)) {
		foreach ($sheetsData as $sheet) {
			$currentSheetId = $sheet->properties->sheetId ?? 0;
			$currentSheetTitle = $sheet->properties->title ?? '';

			if ($currentSheetId === $sheetId) {
				$sheetTitle = $currentSheetTitle;
				break;
			}
		}
	}

	return $sheetTitle;
}


function twerGetSheetIds( string $spreadsheetId ): array {
	$errorTemplate = __( 'The import has failed. Please verify that the %s is correct', 'treweler' );

	try {
		$client = twerGetGoogleClient();

		if ( ! $client['success'] ) {
			throw new \Google\Service\Exception( wp_json_encode( $client ) );
		}

		$service = new Google\Service\Sheets( $client['data'] );

		if ( empty( $spreadsheetId ) ) {
			$error = wp_json_encode( [
				'success' => false,
				'error'   => [
					'code'    => 228,
					'message' => sprintf( $errorTemplate, __( 'Google Spreadsheet ID', 'treweler' ) )
				]
			] );
			throw new \Google\Service\Exception( $error );
		}

		return [ 'success' => true, 'data' => $service->spreadsheets->get( $spreadsheetId ) ];
	}catch ( \Google\Service\Exception $e ) {
		try {
			$message           = json_decode( $e->getMessage(), false, 512, JSON_THROW_ON_ERROR );
			$errorCode         = $message->error->code ?? - 1;
			$errorsDescription = $message->error_description ?? __( 'Something went wrong', 'treweler' );
			switch ( $errorCode ) {
				case 404 :
					$message = [
						'success' => false,
						'error'   => [
							'code'    => $message->error->code,
							'message' => __( 'Requested Spreadsheet ID was not found.', 'treweler' )
						]
					];
					break;
				case 400 :
					$message = [
						'success' => false,
						'error'   => [
							'code'    => $message->error->code,
							'message' => __( 'Google API key not valid. Please pass a valid API key.', 'treweler' )
						]
					];
					break;
				case 403 :
					$message = [
						'success' => false,
						'error'   => [
							'code'    => $message->error->code,
							'message' => __( 'The Google Sheet does not have permission.', 'treweler' )
						]
					];
					break;
				default :
					$message = [
						'success' => false,
						'error'   => [
							'code'    => 1488,
							'message' => $errorsDescription
						]
					];
					break;
			}

			return (array) $message;
		}catch ( \JsonException $e ) {
			return [
				'success' => false,
				'error'   => [
					'code'    => 1488,
					'message' => __( 'The import has failed.', 'treweler' )
				]
			];
		}
	}
}

function twerIsSpreadsheetHasSheets( string $spreadsheetId ): bool {
	$sheetIds = twerGetSheetIds( $spreadsheetId );

	$status = $sheetIds['success'] ?? false;
	$sheets = isset( $sheetIds['data']->sheets ) ?? false;

	return $status && ! empty( $sheets );
}


function twerGetDefaultHeadingColNames(): array {
	return [ 'Name', 'Latitude', 'Longitude', 'Map ID', 'Template ID', 'Category Slug', 'Category ID', 'Image', 'Images', 'Custom Fields', 'Key' ];
}

function twerIsValueCustomFieldKey(string $value) : bool {
	if(empty($value)) return false;
	$maybeFieldKey = twerStrToLower( trim( $value ) );
	return preg_match( "/^field-[\w]{7}(-[a-zA-Z\-]+)?$/i", $maybeFieldKey );
}

function twerIsRowHasKey( array $row ): bool {
	if ( empty( $row ) ) {
		return false;
	}

	$result = array_filter( $row, static function( $v, $k ) {
		return preg_match( '/^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i', $v );
	}, ARRAY_FILTER_USE_BOTH );

	return count( $result ) === 1;
}

function twerIsRowHasLatLon( array $row ): bool {
	if ( empty( $row ) ) {
		return false;
	}

	$result = array_filter( $row, static function( $v, $k ) {
		$patternIsCoords      = '/^[-+]?\d{1,3}(\.\d{5,})?[°]?\s*[NSEWnsew]?$/u';
		$patternIsCoordsRange = '/^[-+]?([1-8]?\d(\.\d{1,})?|90(\.0{1,})?|0\.\d{5,}|[1-9]\d?\.\d{5,}|1[0-7]\d\.\d{5,}|180\.0{1,})[°]?\s*[NSEWnsew]?$/u';

		return preg_match( $patternIsCoordsRange, $v ) === 1 && preg_match( $patternIsCoords, $v ) === 1 && ( ( is_numeric( $v ) && $v >= - 90 && $v <= 90 ) || ( is_numeric( $v ) && $v >= - 180 && $v <= 180 ) );
	}, ARRAY_FILTER_USE_BOTH );

	return count( $result ) >= 2;
}

function twerIsRowAMarker( array $row ): bool {
	if ( empty( $row ) ) {
		return false;
	}

	return twerIsRowHasKey( $row ) || twerIsRowHasLatLon( $row );
}

function twerIsRowAHeading( array $row ): bool {
	if (empty($row)) {
		return false;
	}

	$defaultHeadingColNames = twerGetDefaultHeadingColNames();

	$matches = static function($cellValue) use ($defaultHeadingColNames) {
		foreach ($defaultHeadingColNames as $colName) {
			if (0 === strcasecmp($cellValue, $colName) || twerIsValueCustomFieldKey($cellValue)) {
				return true;
			}
		}
		return false;
	};

	$isHeadingChanceRate = count(array_filter($row, $matches));

	$probablyIsTitleRate = apply_filters('twerSetHeadingChanceRate', 2, $row, $defaultHeadingColNames);

	return $isHeadingChanceRate >= $probablyIsTitleRate;
}

function twerDetermineKeysColumnIndex($tableValues) {

	$tableHeadingIndex = twerGetRowHeadingIndex( $tableValues );
	$tableHeadingCols  = twerGetHeadingCols( $tableValues[ $tableHeadingIndex ] );


	$firstHeadingColIndex = reset( $tableHeadingCols );
	$lastHeadingColIndex  = end( $tableHeadingCols );

	if (twerIsHeadingContainColumn('Key', $tableValues[$tableHeadingIndex])) {
		return $tableHeadingCols['Key'];
	}

	if ($lastHeadingColIndex === 25) {
		if ($firstHeadingColIndex >= 1) {
			return $firstHeadingColIndex - 1;
		}
		$range = range(0, 25);
		$availableNumbers = array_values(array_diff($range, $tableHeadingCols));
		return array_rand(array_flip($availableNumbers), 1);
	}

	do {
		$columnHasValues = false;

		foreach ($tableValues as $row) {
			if (!empty($row[$lastHeadingColIndex])) {
				$columnHasValues = true;
				break;
			}
		}

		if ($columnHasValues) {
			$lastHeadingColIndex++;
		}
	} while ($columnHasValues);

	return $lastHeadingColIndex;
}

function twerGetLastRowIndexFromTable(array $tableValues) : int {
	$tableHeadingIndex = twerGetRowHeadingIndex($tableValues);

	foreach (array_reverse($tableValues, true) as $index => $row) {
		if ($index < $tableHeadingIndex) {
			break;
		}

		if (twerIsRowAMarker($row)) {
			return $index;
		}
	}

	return array_key_last($tableValues);
}

function twerIsHeadingContainColumn( string $name, array $row ): bool {
	if ( empty( $row ) ) {
		return false;
	}

	return array_key_exists( $name, twerGetHeadingCols( $row ) );
}

function twerGetRowSubheadingIndex(array $table ) : int {
	if (empty($table)) {
		return -1;
	}
	$tableHeadingIndex = twerGetRowHeadingIndex( $table );
	$headingRow = $table[ $tableHeadingIndex ];
	$subHeadingRow = $table[ $tableHeadingIndex + 1 ];

	$customFieldsKeys = array_filter($headingRow, static function($cellValue) {
		return twerIsValueCustomFieldKey($cellValue);
	});


	if(!empty($subHeadingRow) && !twerIsRowAMarker($subHeadingRow) && count($customFieldsKeys) >= 1){
		return $tableHeadingIndex + 1;
	}

	return -1;
}

function twerGetRowHeadingIndex( array $table ): int {
	if (empty($table)) {
		return -1;
	}

	foreach ($table as $rowIndex => $row) {
		if (!empty($row) && twerIsRowAHeading($row)) {
			return $rowIndex;
		}
	}

	return -1;
}


function twerGetHeadingCols( array $row ): array {
	if ( empty( $row ) ) {
		return [];
	}

	$defaultHeadingColNames = twerGetDefaultHeadingColNames();

	$headingColIndexes = [];

	foreach ( $defaultHeadingColNames as $headingColName ) {
		foreach ( $row as $cellIndex => $cellValue ) {
			if ( 0 === strcasecmp( $headingColName, $cellValue ) ) {
				$headingColIndexes[ $headingColName ] = $cellIndex;
			} elseif (twerIsValueCustomFieldKey($cellValue)) {
				$headingColIndexes[ $cellValue ] = $cellIndex;
			}
		}
	}

	asort( $headingColIndexes, SORT_NUMERIC );

	return $headingColIndexes;
}

function twerConvertTableNumberToWord( int $number ): string {
	return mb_chr( 65 + ( min( $number, 25 ) ) );
}

function twerIsTableHeadingContainsRequiredCols( array $cols ): bool {
	$requiredCols     = apply_filters( 'twerSetTableHeadingRequiredCols', [ 'Latitude', 'Longitude' ], $cols );
	$validationPoints = 0;
	foreach ( $requiredCols as $requiredCol ) {
		if ( array_key_exists( $requiredCol, $cols ) ) {
			$validationPoints ++;
		}
	}

	return $validationPoints === count( $requiredCols );
}

function twerValidationImportedTable( array $table ): bool {
	$tableIsValid = false;

	$tableHeadingIndex = twerGetRowHeadingIndex( $table );
	if ( $tableHeadingIndex === - 1 ) {
		return false;
	}

	$tableHeadingCols = twerGetHeadingCols( $table[ $tableHeadingIndex ] );
	if ( empty( $tableHeadingCols ) ) {
		return false;
	}

	if ( ! twerIsTableHeadingContainsRequiredCols( $tableHeadingCols ) ) {
		return false;
	}

	return true;
}

function twerGetMarkerDataFromRow( array $row, array $headingCols ): array {
	$data = [];

	foreach ( $headingCols as $headingColName => $headingColIndex ) {
		if ( twerIsValueCustomFieldKey( $headingColName ) ) {
			$headingFieldKey = twerStrToLower( trim( $headingColName ));
			$delimiterPos = strpos( $headingFieldKey, '-', 7 );
			if ( $delimiterPos ) {
				$fieldName =  substr( $headingFieldKey, 0, $delimiterPos );
				$fieldKey  =  substr( $headingFieldKey, $delimiterPos + 1 );
				$data['Custom Fields'][ $fieldName ][ $fieldKey ] = $row[ $headingColIndex ];
			} else {
				$headingColName = twerStrToLower( trim( $headingColName ) );
				$data['Custom Fields'][ $headingColName ] = $row[ $headingColIndex ];
			}
		} else {
			$data[ $headingColName ] = $row[ $headingColIndex ];
		}
	}

	return wp_parse_args( $data, array_fill_keys(  twerGetDefaultHeadingColNames(), '' ) );
}

function twerGetMarkersDataFromTable( array $table, int $start, int $end, int $ignoredRows = 1 ): array {
	$tableHeadingCols = twerGetHeadingCols( $table[ $start ] );

	$markers = [];
	for ( $i = $start + $ignoredRows; $i <= $end; $i ++ ) {
		$row = $table[ $i ];
		if ( ! twerIsRowAMarker( $row ) ) {
			continue;
		}

		$markers[] = twerGetMarkerDataFromRow( $row, $tableHeadingCols );
	}

	return $markers;
}

function twerIsValidImportImage(string $path) : bool {
	if (!twerIsUploadsFileExist($path)) {
		return false;
	}
	return getimagesize(twerGetAbsPathForUploadsFile($path)) !== false;
}


function twerUpdateMarkerCustomField( int $customFieldId, $customFieldValue, int $markerID ) {
	$customFieldType = twerGetCustomFieldType( $customFieldId );
	$customFieldKey  = twerGetCustomFieldKey( $customFieldId );

	switch ( $customFieldType ) {
		case 'text' :
			if ( 'no' === twerIsCustomFieldTextShowAsTitle( $customFieldId ) ) {
				update_post_meta( $markerID, '_treweler_' . $customFieldKey, $customFieldValue );
				update_post_meta( $markerID, '_treweler_' . $customFieldKey . '_lock', trim($customFieldValue) === '' ? 'close' : 'open' );
			}
			break;
		case 'separator' :
			update_post_meta( $markerID, '_treweler_' . $customFieldKey, (int) $customFieldValue );
			update_post_meta( $markerID, '_treweler_' . $customFieldKey . '_lock', trim($customFieldValue) === '' ? 'close' : 'open' );
			break;
		case 'html' :
			update_post_meta( $markerID, '_treweler_' . $customFieldKey, [ 'html' => $customFieldValue ] );
			update_post_meta( $markerID, '_treweler_' . $customFieldKey . '_lock', [ 'html' => trim($customFieldValue) === '' ? 'close' : 'open' ] );
			break;
		case 'number' :
			update_post_meta( $markerID, '_treweler_' . $customFieldKey, [ 'number' => (float) preg_replace( "/[^0-9.]/", "", $customFieldValue ) ] );
			update_post_meta( $markerID, '_treweler_' . $customFieldKey . '_lock', [ 'number' => trim($customFieldValue) === '' ? 'close' : 'open' ] );
			break;
		case 'true_false' :
			$trueFalseImportValue = twerStrToLower( trim( $customFieldValue ) );
			$trueFalseValue       = '';
			if ( $trueFalseImportValue === 'yes' ) {
				$trueFalseValue = [ 'true_false' ];
			}

			update_post_meta( $markerID, '_treweler_' . $customFieldKey, [ 'true_false' => $trueFalseValue ] );
			update_post_meta( $markerID, '_treweler_' . $customFieldKey . '_lock', [ 'true_false' => trim($trueFalseImportValue) === '' ? 'close' : 'open' ] );
			break;
		case 'multiselect' :

			$customFieldValueDefault        = twerGetCustomFieldMultiselectData( $customFieldId );
			$customFieldValueDefaultCleaned = $customFieldValueDefault['valueCleaned'] ?? [];
			$importedValue                  =  trim($customFieldValue) === '' ? [] : explode( ',', $customFieldValue );
			$result                         = [];


			if ( $importedValue && $customFieldValueDefaultCleaned ) {
				foreach ( $importedValue as $item ) {
					$index = array_search( trim( $item ), $customFieldValueDefaultCleaned, true );
					if ( $index !== false ) {
						$result[] = $index;
					}
				}

			}

        update_post_meta( $markerID, '_treweler_' . $customFieldKey, [ 'multiselect' => $result ] );
				update_post_meta( $markerID, '_treweler_' . $customFieldKey . '_lock', [ 'multiselect' => trim($customFieldValue) === '' ? 'close' : 'open'  ] );

			break;
		case 'link' :
		case 'button' :
		case 'email' :
		case 'phone' :
			$text   = $customFieldValue['text'] ?? '';
			$url    = $customFieldValue['url'] ?? '';
			$email  = $customFieldValue['email'] ?? '';
			$number = $customFieldValue['number'] ?? '';
			$target = $customFieldValue['new-tab'] ?? '';

			$newValue        = [];
			$newClosedValues = [];
			if ( ! empty( $text ) ) {
				$newValue['text']        = esc_html( $text );
			}
			$newClosedValues['text'] = trim($text) === '' ? 'close' : 'open';

			if ( $customFieldType === 'link' || $customFieldType === 'button' ) {
        $newValue['url']        = esc_url( $url );
				$newClosedValues['url'] = trim($url) === '' ? 'close' : 'open';



					$metaTargetKey = 'target';
					if ( $customFieldType === 'button' ) {
						$metaTargetKey = 'btn_target';
					}

					$targetValue       = '';
					$targetImportValue = twerStrToLower( trim( $target ) );
					if ( $targetImportValue === 'yes' ) {
						$targetValue = [ $metaTargetKey ];
					}

					$newValue[ $metaTargetKey ]        = $targetValue;
					$newClosedValues[ $metaTargetKey ] =  trim($target) === '' ? 'close' : 'open';

			} elseif ( $customFieldType === 'email' ) {

        $newValue['url']        = sanitize_email( $email );
				$newClosedValues['url'] = trim($email) === '' ? 'close' : 'open';
			} elseif ( $customFieldType === 'phone' ) {

        $newValue['url']        = twerSanitizePhoneNumber( $number );
				$newClosedValues['url'] = trim($number) === '' ? 'close' : 'open';
			}


			update_post_meta( $markerID, '_treweler_' . $customFieldKey, $newValue );
			update_post_meta( $markerID, '_treweler_' . $customFieldKey . '_lock', $newClosedValues );
			break;
	}
}

function twerUpdateMarkerCustomFields( array $customFields, int $markerID ) {
	foreach ( $customFields as $customFieldKey => $customFieldValue ) {
		$customFieldId = twerGetCustomFieldIdBy( 'key', $customFieldKey );

		if ( $customFieldId ) {
			if ( 'no' === twerIsMarkerHasCustomFieldId( $markerID, $customFieldId ) ) {
				continue;
			}

			twerUpdateMarkerCustomField( $customFieldId, $customFieldValue, $markerID );
		}
	}
}
