<?php
/**
 * Treweler Pro setup
 *
 * @package Treweler
 * @since   1.14
 */

namespace TrewelerPro;

defined( 'ABSPATH' ) || exit;

/**
 * Main Treweler Pro Class.
 *
 * @class Treweler
 */
final class Treweler {

  /**
   * The single instance of the class.
   *
   * @var \TrewelerPro\Treweler|null
   * @since 1.14
   */
  protected static ?Treweler $_instance = null;

  /**
   * Main Treweler Pro Instance.
   *
   * Ensures only one instance of Treweler Pro is loaded or can be loaded.
   *
   * @return Treweler Pro - Main instance.
   * @see TWER_PRO()
   * @since 1.14
   * @static
   */
  public static function instance(): Treweler {
    if ( is_null( self::$_instance ) ) {
      self::$_instance = new self();
    }

    return self::$_instance;
  }

  /**
   * Treweler Pro Constructor.
   */
  public function __construct() {
    $this->includes();
  }

  /**
   * Cloning is forbidden.
   *
   * @since 1.14
   */
  public function __clone() {
    twer_doing_it_wrong( __FUNCTION__, esc_html__( 'Cloning is forbidden.', 'treweler' ), '1.14' );
  }

  /**
   * Unserializing instances of this class is forbidden.
   *
   * @since 1.14
   */
  public function __wakeup() {
    twer_doing_it_wrong( __FUNCTION__, esc_html__( 'Unserializing instances of this class is forbidden.', 'treweler' ),
      '1.14' );
  }


  /**
   * Include required core files used in admin and on the frontend.
   */
  public function includes(): void {
    /**
     * Core classes.
     */
	  include_once TWER_ABSPATH . 'pro/includes/class-twer-mime-types.php';
    include_once TWER_ABSPATH . 'pro/includes/class-twer-post-types.php';
    include_once TWER_ABSPATH . 'pro/includes/class-twer-ajax.php';
    include_once TWER_ABSPATH . 'pro/includes/class-twer-taxonomy.php';

    if (is_admin() ) {
      include_once TWER_ABSPATH . 'pro/includes/admin/class-twer-admin.php';
    }

    if ( ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' ) ) {
      $this->frontend_includes();
    }
  }


  /**
   * Include required frontend files.
   */
  public function frontend_includes(): void {
    include_once TWER_ABSPATH . 'pro/includes/twer-map-functions.php';
    include_once TWER_ABSPATH . 'pro/includes/class-twer-map.php';
  }
}
