<?php
/**
 * Treweler TWER_AJAX. AJAX Event Handlers.
 *
 * @class   TWER_AJAX
 * @package Treweler\Classes
 */

namespace TrewelerPro;

use Google;

defined( 'ABSPATH' ) || exit;

/**
 * TWER_Ajax class.
 */
class TWER_AJAX {

	/**
	 * Hook in ajax handlers.
	 */
	public static function init() {
		self::add_ajax_events();
	}

	/**
	 * Hook in methods - uses WordPress ajax handlers (admin-ajax).
	 */
	public static function add_ajax_events() {
		$ajax_events_nopriv = array();

		foreach ( $ajax_events_nopriv as $ajax_event ) {
			add_action( 'wp_ajax_twer_' . $ajax_event, array( __CLASS__, $ajax_event ) );
			add_action( 'wp_ajax_nopriv_twer_' . $ajax_event, array( __CLASS__, $ajax_event ) );

			// TW AJAX can be used for frontend ajax requests.
			add_action( 'wc_ajax_' . $ajax_event, array( __CLASS__, $ajax_event ) );
		}

		$ajax_events = array(
			'import_markers',
			'get_sheet_ids',
			'get_import_progress'
		);

		foreach ( $ajax_events as $ajax_event ) {
			add_action( 'wp_ajax_twer_' . $ajax_event, array( __CLASS__, $ajax_event ) );
		}

		$ajax_private_events = array();

		foreach ( $ajax_private_events as $ajax_event ) {
			add_action(
				'wp_ajax_twer_' . $ajax_event,
				function() use ( $ajax_event ) {
					call_user_func( array( __CLASS__, $ajax_event ) );
				}
			);
		}
	}

	public static function get_import_progress() {
		check_ajax_referer( 'update-post_' . $_POST['post_ID'] );

		$spreadsheetId   = $_POST['_treweler_spreadsheet_id'] ?? '';
		$sheetId         = isset( $_POST['_treweler_sheet_id'] ) ? (int) $_POST['_treweler_sheet_id'] : 0;
		$importID        = isset( $_POST['post_ID'] ) ? (int) $_POST['post_ID'] : 0;
		$current_user_id = get_current_user_id();


		$transientNameImportProgress = "treweler_google_import_progress_{$importID}-{$current_user_id}-{$sheetId}-{$spreadsheetId}";

		wp_send_json( get_transient( $transientNameImportProgress ) );
	}

	public static function get_sheet_ids() {
		@ini_set( 'memory_limit', '4000M' );
		if ( function_exists( 'gc_enable' ) ) {
			gc_enable(); // phpcs:ignore PHPCompatibility.FunctionUse.NewFunctions.gc_enableFound
		}
		twer_set_time_limit( 0 );
		@ob_flush();
		@flush();
		check_ajax_referer( 'update-post_' . $_POST['post_ID'] );
		$spreadsheetId = $_POST['_treweler_spreadsheet_id'] ?? '';
		wp_send_json( twerGetSheetIds( $spreadsheetId ) );
	}

	public static function import_markers() {
		@ini_set( 'memory_limit', '4000M' );
		if ( function_exists( 'gc_enable' ) ) {
			gc_enable(); // phpcs:ignore PHPCompatibility.FunctionUse.NewFunctions.gc_enableFound
		}
		twer_set_time_limit( 0 );
		@ob_flush();
		@flush();


		check_ajax_referer( 'update-post_' . $_POST['post_ID'] );
		$errorTemplate = __( 'The import has failed. Please verify that the %s is correct', 'treweler' );

		$current_user_id = get_current_user_id();

		$spreadsheetId = $_POST['_treweler_spreadsheet_id'] ?? '';
		$sheetId       = isset( $_POST['_treweler_sheet_id'] ) ? (int) $_POST['_treweler_sheet_id'] : 0;
		$ignoredRows   = isset( $_POST['_treweler_top_rows_ignored'] ) ? (int) $_POST['_treweler_top_rows_ignored'] : 1;
		$rule          = $_POST['_treweler_removal_rule'] ?? 'dont_rm_markers';
		$importID      = isset( $_POST['post_ID'] ) ? (int) $_POST['post_ID'] : 0;

		$transientNameImportProgress = "treweler_google_import_progress_{$importID}-{$current_user_id}-{$sheetId}-{$spreadsheetId}";
		delete_transient( $transientNameImportProgress );
		set_transient( $transientNameImportProgress, [ 'percentage' => 0, 'imported' => 0, 'updated' => 0, 'removed' => 0, 'error' => false, ], 10 * MINUTE_IN_SECONDS );


		if ( $ignoredRows <= 0 ) {
			$ignoredRows = 1;
		}

		$totalRemovedMarkers = 0;
		$totalUpdatedMarkers = 0;
		$totalCreatedMarkers = 0;

		try {
			$client = twerGetGoogleClient();

			if ( ! $client['success'] ) {
				throw new \Google\Service\Exception( wp_json_encode( $client ) );
			}

			$service = new Google\Service\Sheets( $client['data'] );

			if ( empty( $spreadsheetId ) ) {
				$error = wp_json_encode( [
					'success' => false,
					'error'   => [
						'code'    => 228,
						'message' => sprintf( $errorTemplate, __( 'Google Spreadsheet ID', 'treweler' ) )
					]
				] );
				throw new \Google\Service\Exception( $error );
			}

			$sheetTitle = twerGetSheetTitleById( $sheetId, $spreadsheetId );

			$tableValues = [];
			$response    = [];
			if ( ! empty( $sheetTitle ) ) {
				$range       = $sheetTitle;
				$response    = $service->spreadsheets_values->get( $spreadsheetId, $range );
				$tableValues = $response->getValues();
			}
			if ( empty( $tableValues ) ) {
				$error = wp_json_encode( [
					'success' => false,
					'error'   => [
						'code'    => 228,
						'message' => __( 'Google Spreadsheet Is Empty.', 'treweler' )
					]
				] );
				throw new \Google\Service\Exception( $error );
			}


			$tableHeadingIndex = twerGetRowHeadingIndex( $tableValues );


			if ( $tableHeadingIndex === - 1 ) {
				$error = wp_json_encode( [
					'success' => false,
					'error'   => [
						'code'    => 228,
						'message' => __( 'Table header is missing. Add a header and try again.', 'treweler' )
					]
				] );
				throw new \Google\Service\Exception( $error );
			}

			$tableHeadingRow = $tableValues[ $tableHeadingIndex ];

			$tableSubheadingIndex = twerGetRowSubheadingIndex( $tableValues );


			$tableHeadingCols = twerGetHeadingCols( $tableHeadingRow );


			$firstHeadingColIndex = reset( $tableHeadingCols );
			$lastHeadingColIndex  = end( $tableHeadingCols );

			$lastMarkerRowIndex = twerGetLastRowIndexFromTable( $tableValues );


			$keysColumnIndex = 0;
			$values1         = [];
			if ( twerIsHeadingContainColumn( 'Key', $tableHeadingRow ) ) {
				$keysColumnIndex = $tableHeadingCols['Key'];
			} else {

				$keysColumnIndex = twerDetermineKeysColumnIndex( $tableValues );

				$keysColumnWord  = twerConvertTableNumberToWord( $keysColumnIndex );
				$keyColumnsRange = sprintf( '%s!%s%d', $sheetTitle, $keysColumnWord, $tableHeadingIndex + 1 );

				$body = new \Google_Service_Sheets_ValueRange( [
					'values' => [
						[ 'Key' ]
					]
				] );

				$service->spreadsheets_values->update( $spreadsheetId, $keyColumnsRange, $body, [
					'valueInputOption' => 'USER_ENTERED'
				] );
			}


			$a = [];
			for ( $i = $tableHeadingIndex; $i <= $lastMarkerRowIndex; $i ++ ) {
				if ( $i === $tableHeadingIndex ) {
					$tableValues[ $i ][ $keysColumnIndex ] = 'Key';
					continue;
				}
				if ( $i === $tableSubheadingIndex || empty( $tableValues[ $i ] ) ) {
					$tableValues[ $i ][ $keysColumnIndex ] = '';
					continue;
				}
				if ( ! twerIsRowHasKey( $tableValues[ $i ] ) ) {
					$key                                   = wp_generate_uuid4();
					$a[ $i + 1 ]                           = $key;
					$tableValues[ $i ][ $keysColumnIndex ] = $key;
				}
			}

			$new      = [];
			$start    = null;
			$end      = null;
			$subarray = [];

			foreach ( $a as $index => $value ) {
				if ( $start === null ) {
					$start = $index;
					$end   = $index;
				} elseif ( $index === $end + 1 ) {
					$end = $index;
				} else {

					$keysColumnWord = twerConvertTableNumberToWord( $keysColumnIndex );

					$key         = $start === $end ? sprintf( '%s!%s%d', $sheetTitle, $keysColumnWord, $start ) : sprintf( '%1$s!%2$s%3$d:%2$s%4$d', $sheetTitle, $keysColumnWord, $start, $end );
					$new[ $key ] = $subarray;
					$subarray    = [];
					$start       = $index;
					$end         = $index;
				}

				$subarray[] = [ $value ];
			}

			if ( $start !== null ) {
				$keysColumnWord = twerConvertTableNumberToWord( $keysColumnIndex );
				$key            = $start === $end ? sprintf( '%s!%s%d', $sheetTitle, $keysColumnWord, $start ) : sprintf( '%1$s!%2$s%3$d:%2$s%4$d', $sheetTitle, $keysColumnWord, $start, $end );
				$new[ $key ]    = $subarray;
			}


			if ( ! empty( $new ) ) {
				foreach ( $new as $markersKeysRange => $markerKeysValues ) {
					$body = new \Google_Service_Sheets_ValueRange( [
						'values' => $markerKeysValues
					] );
					$service->spreadsheets_values->update( $spreadsheetId, $markersKeysRange, $body, [
						'valueInputOption' => 'USER_ENTERED'
					] );
				}
			}


			$requests = [
				new \Google_Service_Sheets_Request( [
					'repeatCell' => [
						'range'  => [
							'sheetId'          => $sheetId,
							'startRowIndex'    => $tableHeadingIndex,
							'endRowIndex'      => $tableHeadingIndex + 1,
							'startColumnIndex' => $keysColumnIndex,
							'endColumnIndex'   => $lastHeadingColIndex + 1
						],
						'cell'   => [
							'userEnteredFormat' => [
								'textFormat' => [
									'bold' => true
								]
							]
						],
						'fields' => 'userEnteredFormat.textFormat.bold'
					]
				] )
			];

			$batchUpdateRequest = new \Google_Service_Sheets_BatchUpdateSpreadsheetRequest( [
				'requests' => $requests
			] );

			$response1 = $service->spreadsheets->batchUpdate( $spreadsheetId, $batchUpdateRequest );


			$markersData = twerGetMarkersDataFromTable( $tableValues, $tableHeadingIndex, $lastMarkerRowIndex, $ignoredRows );

			if ( empty( $markersData ) ) {
				$error = wp_json_encode( [
					'success' => false,
					'error'   => [
						'code'    => 228,
						'message' => __( 'There are no markers to export.', 'treweler' )
					]
				] );
				throw new \Google\Service\Exception( $error );
			}


			$markersKeys = array_column( $markersData, 'Key' );


			$posts_not_in_keys = twerGetMarkersForDeletionByImportRule( $rule, $markersKeys );

			$totalSteps  = count( $posts_not_in_keys ) + count( $markersData );
			$currentStep = 0;

			if ( ! empty( $posts_not_in_keys ) ) {
				foreach ( $posts_not_in_keys as $post_id ) {

					wp_delete_post( $post_id, true );
					$totalRemovedMarkers ++;

					$currentStep ++;
					$percentage = round( ( $currentStep / $totalSteps ) * 100 );

					set_transient( $transientNameImportProgress, [ 'percentage' => $percentage, 'imported' => $totalCreatedMarkers, 'updated' => $totalUpdatedMarkers, 'removed' => $totalRemovedMarkers, 'error' => false ], 10 * MINUTE_IN_SECONDS );
				}
			}


			$posts_with_keys = twerGetImportedMarkersByKeys( $markersKeys );

			$uncategorizedId = (int) get_option( 'default_map_category' );
			if ( empty( $uncategorizedId ) && ! is_numeric( $uncategorizedId ) ) {
				$uncategorized   = get_term_by( 'slug', 'uncategorized-map', 'map-category' );
				$uncategorizedId = isset( $uncategorized->term_id ) ? (int) $uncategorized->term_id : 0;
			}

			$importedCf = [];


			foreach ( $markersData as $marker ) {

				$metaRecords = [
					'_treweler_google_import_id' => $importID,
				];


				$taxonomy = 'map-category';


				if ( in_array( $marker['Key'], $posts_with_keys, true ) ) {
					// Post already exist, update it
					$markerID = array_search( $marker['Key'], $posts_with_keys, true );


					$latLon    = get_post_meta( $markerID, '_treweler_marker_latlng', true );
					$latitude  = $latLon[0];
					$longitude = $latLon[1];

					if ( twerIsHeadingContainColumn( 'Latitude', $tableHeadingRow ) ) {
						if ( ! empty( $marker['Latitude'] ) ) {
							$latitude = $marker['Latitude'];
						}
					}
					if ( twerIsHeadingContainColumn( 'Longitude', $tableHeadingRow ) ) {
						if ( ! empty( $marker['Longitude'] ) ) {
							$longitude = $marker['Longitude'];
						}
					}

					$metaRecords['_treweler_marker_latlng'] = [ $latitude, $longitude ];

					if ( twerIsHeadingContainColumn( 'Map ID', $tableHeadingRow ) ) {
						if ( empty( $marker['Map ID'] ) ) {
							$metaRecords['_treweler_marker_map_id'] = '';
						} else {
							$metaRecords['_treweler_marker_map_id'] = array_map( static function( $mapId ) {
								return trim( $mapId );
							}, explode( ',', $marker['Map ID'] ) );
						}
					}

					if ( twerIsHeadingContainColumn( 'Template ID', $tableHeadingRow ) ) {
						if ( empty( $marker['Template ID'] ) ) {
							$metaRecords['_treweler_templates'] = 'none';
						} else {
							$metaRecords['_treweler_templates'] = (int) $marker['Template ID'];
						}
					}

					$ignoreTemplateMeta = [];
					if ( twerIsHeadingContainColumn( 'Image', $tableHeadingRow ) || twerIsHeadingContainColumn( 'Images', $tableHeadingRow ) ) {

						$imagePath   = trim( $marker['Image'] ) ?? '';
						$imagesPath  = trim( $marker['Images'] ) ?? '';
						$imageArray  = [];
						$imagesArray = [];
						if ( ! empty( $imagePath ) ) {

							$imageArray = array_filter( array_map( static function( $path ) {
								$path   = trim( $path );
								$result = null;
								if ( twerIsValidImportImage( $path ) ) {
									$attachmentId = twerGetUploadsFileIdFromPath( $path );
									$result       = ! empty( $attachmentId ) ? $attachmentId : null;
								}

								return $result;
							}, explode( ',', $imagePath ) ), static function( $value ) {
								return ! is_null( $value );
							} );
						}
						if ( ! empty( $imagesPath ) ) {
							$imagesArray = array_filter( array_map( static function( $path ) {
								$path   = trim( $path );
								$result = null;
								if ( twerIsValidImportImage( $path ) ) {
									$attachmentId = twerGetUploadsFileIdFromPath( $path );
									$result       = ! empty( $attachmentId ) ? $attachmentId : null;
								}

								return $result;
							}, explode( ',', $imagesPath ) ), static function( $value ) {
								return ! is_null( $value );
							} );
						}

						$markerImages = array_unique( array_merge( $imageArray, $imagesArray ) );

						if ( ! empty( $markerImages ) ) {
							$ignoreTemplateMeta[]                      = 'popup_image';
							$metaRecords['_treweler_popup_image']      = implode( ',', $markerImages );
							$metaRecords['_treweler_popup_image_lock'] = 'open';
						} else {
							$metaRecords['_treweler_popup_image']      = '';
							$metaRecords['_treweler_popup_image_lock'] = 'close';
						}
					} else {
						$popupImage     = get_post_meta( $markerID, '_treweler_popup_image', true );
						$popupImageLock = get_post_meta( $markerID, '_treweler_popup_image_lock', true );

						if ( ! empty( $popupImage ) || $popupImageLock === 'open' ) {
							$ignoreTemplateMeta[] = 'popup_image';
						}
					}


					$terms = [];

					if ( twerIsHeadingContainColumn( 'Category ID', $tableHeadingRow ) ) {
						if ( empty( $marker['Category ID'] ) ) {
							$terms = [
								$uncategorizedId
							];
						} else {
							$categoriesIDs = array_map( static function( $categoryId ) {
								return trim( $categoryId );
							}, explode( ',', $marker['Category ID'] ) );

							foreach ( $categoriesIDs as $termId ) {
								$term = get_term_by( 'term_id', $termId, $taxonomy );
								if ( empty( $term ) ) {
									continue;
								}
								if ( in_array( $term->term_id, $terms, true ) ) {
									continue;
								}

								$terms[] = $term->term_id;

							}
						}
					}

					if ( twerIsHeadingContainColumn( 'Category Slug', $tableHeadingRow ) ) {
						if ( empty( $marker['Category Slug'] ) ) {
							$terms = [
								$uncategorizedId
							];
						} else {
							$categoriesSlugs = array_map( static function( $categoryId ) {
								return trim( $categoryId );
							}, explode( ',', $marker['Category Slug'] ) );

							foreach ( $categoriesSlugs as $termSlug ) {
								$term = get_term_by( 'slug', $termSlug, $taxonomy );
								if ( empty( $term ) ) {
									continue;
								}
								if ( in_array( $term->term_id, $terms, true ) ) {
									continue;
								}

								$terms[] = $term->term_id;

							}
						}
					}

					$updateData = [
						'ID'         => $markerID,
						'meta_input' => $metaRecords,
					];

					if ( twerIsHeadingContainColumn( 'Name', $tableHeadingRow ) ) {
						$updateData['post_title'] = empty( $marker['Name'] ) ? '' : sanitize_text_field( $marker['Name'] );
					}

					wp_update_post( wp_slash( $updateData ), true );


					if ( ! empty( $markerID ) && ! empty( $terms ) ) {
						wp_set_object_terms( $markerID, $terms, $taxonomy );
					}


					if ( twerIsHeadingContainColumn( 'Template ID', $tableHeadingRow ) ) {
						if ( ! empty( $marker['Template ID'] ) ) {
							twerUpdateMetaForMarkerWithTemplate( $markerID, $marker['Template ID'], $ignoreTemplateMeta );
						}
					}


					if(!empty($marker['Custom Fields'])) {
						twerUpdateMarkerCustomFields( $marker['Custom Fields'], $markerID );
					}


					$totalUpdatedMarkers ++;
				} else {
					// Post not exist, crate new one
					$metaRecords['_treweler_google_import_marker_key'] = $marker['Key'];

					$markerID = 0;

					if ( ! empty( $marker['Latitude'] ) && ! empty( $marker['Longitude'] ) ) {


						$metaRecords['_treweler_marker_latlng'] = [ $marker['Latitude'], $marker['Longitude'] ];


						if ( empty( $marker['Map ID'] ) ) {
							$metaRecords['_treweler_marker_map_id'] = '';
						} else {
							$metaRecords['_treweler_marker_map_id'] = array_map( static function( $mapId ) {
								return trim( $mapId );
							}, explode( ',', $marker['Map ID'] ) );
						}

						if ( empty( $marker['Template ID'] ) ) {
							$metaRecords['_treweler_templates'] = 'none';
						} else {
							$metaRecords['_treweler_templates'] = (int) $marker['Template ID'];
						}


						$ignoreTemplateMeta = [];
						if ( twerIsHeadingContainColumn( 'Image', $tableHeadingRow ) || twerIsHeadingContainColumn( 'Images', $tableHeadingRow ) ) {

							$imagePath   = trim( $marker['Image'] ) ?? '';
							$imagesPath  = trim( $marker['Images'] ) ?? '';
							$imageArray  = [];
							$imagesArray = [];
							if ( ! empty( $imagePath ) ) {

								$imageArray = array_filter( array_map( static function( $path ) {
									$path   = trim( $path );
									$result = null;
									if ( twerIsValidImportImage( $path ) ) {
										$attachmentId = twerGetUploadsFileIdFromPath( $path );
										$result       = ! empty( $attachmentId ) ? $attachmentId : null;
									}

									return $result;
								}, explode( ',', $imagePath ) ), static function( $value ) {
									return ! is_null( $value );
								} );
							}
							if ( ! empty( $imagesPath ) ) {
								$imagesArray = array_filter( array_map( static function( $path ) {
									$path   = trim( $path );
									$result = null;
									if ( twerIsValidImportImage( $path ) ) {
										$attachmentId = twerGetUploadsFileIdFromPath( $path );
										$result       = ! empty( $attachmentId ) ? $attachmentId : null;
									}

									return $result;
								}, explode( ',', $imagesPath ) ), static function( $value ) {
									return ! is_null( $value );
								} );
							}

							$markerImages = array_unique( array_merge( $imageArray, $imagesArray ) );

							if ( ! empty( $markerImages ) ) {
								$ignoreTemplateMeta[]                      = 'popup_image';
								$metaRecords['_treweler_popup_image']      = implode( ',', $markerImages );
								$metaRecords['_treweler_popup_image_lock'] = 'open';
							} else {
								$metaRecords['_treweler_popup_image']      = '';
								$metaRecords['_treweler_popup_image_lock'] = 'close';
							}
						}


						$terms = [];

						if ( ! empty( $marker['Category ID'] ) ) {
							$categoriesIDs = array_map( static function( $categoryId ) {
								return trim( $categoryId );
							}, explode( ',', $marker['Category ID'] ) );

							foreach ( $categoriesIDs as $termId ) {
								$term = get_term_by( 'term_id', $termId, $taxonomy );
								if ( empty( $term ) ) {
									continue;
								}
								if ( in_array( $term->term_id, $terms, true ) ) {
									continue;
								}

								$terms[] = $term->term_id;

							}
						}

						if ( ! empty( $marker['Category Slug'] ) ) {
							$categoriesSlugs = array_map( static function( $categoryId ) {
								return trim( $categoryId );
							}, explode( ',', $marker['Category Slug'] ) );

							foreach ( $categoriesSlugs as $termSlug ) {
								$term = get_term_by( 'slug', $termSlug, $taxonomy );
								if ( empty( $term ) ) {
									continue;
								}
								if ( in_array( $term->term_id, $terms, true ) ) {
									continue;
								}

								$terms[] = $term->term_id;

							}
						}

						$insertData = [
							'post_title'  => sanitize_text_field( $marker['Name'] ),
							'post_status' => 'publish',
							'post_type'   => 'marker',
							'meta_input'  => $metaRecords,
						];

						$markerID = wp_insert_post( wp_slash( $insertData ), true );


						if ( empty( $terms ) ) {
							$terms = [ $uncategorizedId ];
						}

						if ( ! empty( $markerID ) && ! empty( $terms ) ) {
							wp_set_object_terms( $markerID, $terms, $taxonomy );
						}


						if ( ! empty( $marker['Template ID'] ) ) {
							twerUpdateMetaForMarkerWithTemplate( $markerID, $marker['Template ID'], $ignoreTemplateMeta );

							if(!empty($marker['Custom Fields'])) {
								twerUpdateMarkerCustomFields( $marker['Custom Fields'], $markerID );
							}
						}

						$totalCreatedMarkers ++;
					}


				}


				$currentStep ++;
				$percentage = round( ( $currentStep / $totalSteps ) * 100 );

				set_transient( $transientNameImportProgress, [ 'percentage' => $percentage, 'imported' => $totalCreatedMarkers, 'updated' => $totalUpdatedMarkers, 'removed' => $totalRemovedMarkers, 'error' => false ], 10 * MINUTE_IN_SECONDS );

			}


			update_post_meta( $importID, '_treweler_import_last_status', [ 'imported' => $totalCreatedMarkers, 'updated' => $totalUpdatedMarkers, 'removed' => $totalRemovedMarkers ] );
			update_post_meta( $importID, '_treweler_import_last_import_date', time() );


			wp_send_json( [
				'data'       => [
					'$importedCf'           => $importedCf,
					'$response'             => [
						'sheetId'          => $sheetId,
						'startRowIndex'    => $tableHeadingIndex,
						'endRowIndex'      => $tableHeadingIndex + 1,
						'startColumnIndex' => $keysColumnIndex,
						'endColumnIndex'   => $lastHeadingColIndex + 1
					],
					'$markersData'          => $markersData,
					'$keysRangeFill'        => $new,
					'$tableSubheadingIndex' => $tableSubheadingIndex,
					'$keysColumnIndex'      => $keysColumnIndex,
					'$tableValues'          => $tableValues,
					'$tableHeadingIndex'    => $tableHeadingIndex,
					'$tableHeadingCols'     => $tableHeadingCols
				],
				'success'    => true,
				'percentage' => 100,
				'imported'   => $totalCreatedMarkers,
				'updated'    => $totalUpdatedMarkers,
				'removed'    => $totalRemovedMarkers,
				'error'      => false,
				'message'    => __( 'Marker import has been successfully completed.', 'treweler' ),
			] );

		}catch ( \Google\Service\Exception $e ) {
			set_transient( $transientNameImportProgress, [ 'percentage' => 100, 'imported' => $totalCreatedMarkers, 'updated' => $totalUpdatedMarkers, 'removed' => $totalRemovedMarkers, 'error' => true ], 10 * MINUTE_IN_SECONDS );
			try {
				$message = json_decode( $e->getMessage(), false, 512, JSON_THROW_ON_ERROR );

				$errorCode   = $message->error->code ?? 0;
				$errorStatus = $message->error->status ?? '';

				if ( $errorStatus === "PERMISSION_DENIED" && $errorCode === 403 ) {
					$message->error->message = __( 'The caller does not have permission. Check that your Google Spreadsheet is allowed access to anyone with the link with the editor permissions.', 'treweler' );
				}

				wp_send_json( (array) $message );
			}catch ( \JsonException $e ) {
				wp_send_json( [
					'success' => false,
					'error'   => [
						'code'    => 1488,
						'message' => __( 'The import has failed.', 'treweler' )
					]
				] );
			}
		}

		wp_die();
	}

}

TWER_AJAX::init();
