<?php
/**
 * PRO Custom Post Types
 * Registers post types and taxonomies for PRO version.
 *
 * @package Treweler/Classes/PostTypes
 * @version 1.14
 */

namespace TrewelerPro;

defined( 'ABSPATH' ) || exit;


/**
 * Post types Class.
 */
class TWER_Post_Types {

  /**
   * Hook in methods.
   */
  public static function init() {
    add_action( 'treweler_register_taxonomy', [ __CLASS__, 'register_taxonomies' ], 5 );
    add_action( 'treweler_register_post_type', [ __CLASS__, 'register_post_types' ], 5 );
    add_filter( 'treweler_term_messages', [ __CLASS__, 'updated_term_messages' ], 5 );
    add_filter('twerDisabledGutenbergPostTypes', [ __CLASS__, 'gutenberg_can_edit_post_type' ], 5, 3);
  }

  /**
   * Register core taxonomies.
   */
  public static function register_taxonomies() {
    if ( ! is_blog_installed() ) {
      return;
    }

    if ( taxonomy_exists( 'map-category' ) ) {
      return;
    }

    $args = [
      'labels'            => [
        'name'                       => esc_html_x( 'Categories', 'Taxonomy General Name', 'treweler' ),
        'singular_name'              => esc_html_x( 'Category', 'Taxonomy Singular Name', 'treweler' ),
        'menu_name'                  => esc_html__( 'Categories', 'treweler' ),
        'all_items'                  => esc_html__( 'All Items', 'treweler' ),
        'parent_item'                => esc_html__( 'Parent Item', 'treweler' ),
        'parent_item_colon'          => esc_html__( 'Parent Item:', 'treweler' ),
        'new_item_name'              => esc_html__( 'New Item Name', 'treweler' ),
        'add_new_item'               => esc_html__( 'Add New Item', 'treweler' ),
        'edit_item'                  => esc_html__( 'Edit Item', 'treweler' ),
        'update_item'                => esc_html__( 'Update Item', 'treweler' ),
        'view_item'                  => esc_html__( 'View Item', 'treweler' ),
        'separate_items_with_commas' => esc_html__( 'Separate items with commas', 'treweler' ),
        'add_or_remove_items'        => esc_html__( 'Add or remove items', 'treweler' ),
        'choose_from_most_used'      => esc_html__( 'Choose from the most used', 'treweler' ),
        'popular_items'              => esc_html__( 'Popular Items', 'treweler' ),
        'search_items'               => esc_html__( 'Search Items', 'treweler' ),
        'not_found'                  => esc_html__( 'Not Found', 'treweler' ),
        'no_terms'                   => esc_html__( 'No items', 'treweler' ),
        'items_list'                 => esc_html__( 'Items list', 'treweler' ),
        'items_list_navigation'      => esc_html__( 'Items list navigation', 'treweler' ),
      ],
      'hierarchical'      => true,
      'public'            => true,
      'show_ui'           => true,
      'show_admin_column' => true,
      'show_in_nav_menus' => true,
      'query_var'         => true,
      'show_tagcloud'     => false,
      'show_in_menu'      => true,
      'rewrite'           => false,
      'show_in_rest'      => false,
    ];
    register_taxonomy( 'map-category',
      apply_filters( 'treweler_register_taxonomy_map_post_type', [ 'map', 'marker', 'route' ] ),
      apply_filters( 'treweler_register_taxonomy_map_args', $args ) );
  }


  /**
   * Register core post types.
   */
  public static function register_post_types() {
    if ( ! is_blog_installed() ||
         post_type_exists( 'twer-templates' ) ||
         post_type_exists( 'twer-custom-fields' ) ||
         post_type_exists( 'twer-shapes' ) ||
         post_type_exists( 'treweler-import' )
    ) {
      return;
    }

    register_post_type(
      'twer-shapes',
      apply_filters(
        'treweler_register_post_type_shapes',
        [
          'labels'              => [
            'name'                  => esc_html_x( 'Shapes', 'Post type general name', 'treweler' ),
            'singular_name'         => esc_html_x( 'Shape', 'Post type singular name', 'treweler' ),
            'add_new'               => esc_html__( 'Add New', 'treweler' ),
            'add_new_item'          => esc_html__( 'Add New Shape', 'treweler' ),
            'edit_item'             => esc_html__( 'Edit Shape', 'treweler' ),
            'new_item'              => esc_html__( 'New Shape', 'treweler' ),
            'view_item'             => esc_html__( 'View Shape', 'treweler' ),
            'all_items'             => esc_html__( 'Shapes', 'treweler' ),
            'search_items'          => esc_html__( 'Search Shapes', 'treweler' ),
            'not_found'             => esc_html__( 'No Shapes found.', 'treweler' ),
            'not_found_in_trash'    => esc_html__( 'No Shapes found in Trash.', 'treweler' ),
            'menu_name'             => esc_html_x( 'Treweler', 'Admin menu name', 'treweler' ),
            'name_admin_bar'        => esc_html_x( 'Shape', 'Add New on Toolbar', 'treweler' ),
            'parent_item_colon'     => esc_html__( 'Parent Shapes:', 'treweler' ),
            'insert_into_item'      => esc_html__( 'Insert into Shape', 'treweler' ),
            'uploaded_to_this_item' => esc_html__( 'Uploaded to this Shape', 'treweler' ),
            'filter_items_list'     => esc_html__( 'Filter Shapes list', 'treweler' ),
            'items_list_navigation' => esc_html__( 'Shapes list navigation', 'treweler' ),
            'items_list'            => esc_html__( 'Shapes list', 'treweler' ),
          ],
          'description'         => esc_html__( 'Adds Shapes for maps.', 'treweler' ),
          'public'              => false,
          'publicly_queryable'  => false,
          'query_var'           => false,
          'show_ui'             => true,
          'show_in_menu'        => current_user_can( 'edit_treweler' ) ? 'treweler' : true,
          'show_in_nav_menus'   => false,
          'capability_type'     => 'treweler',
          'has_archive'         => false,
          'hierarchical'        => false,
          'can_export'          => true,
          'menu_position'       => null,
          'rewrite'             => false,
          'exclude_from_search' => true,
          //'taxonomies'          => [ 'map-category' ],
          'supports'            => [ 'title' ],
        ]
      )
    );

    register_post_type(
      'twer-custom-fields',
      apply_filters(
        'treweler_register_post_type_custom_fields',
        [
          'labels'              => [
            'name'                  => esc_html_x( 'Custom Fields', 'Post type general name', 'treweler' ),
            'singular_name'         => esc_html_x( 'Custom Field', 'Post type singular name', 'treweler' ),
            'add_new'               => esc_html__( 'Add New', 'treweler' ),
            'add_new_item'          => esc_html__( 'Add New Custom Field', 'treweler' ),
            'edit_item'             => esc_html__( 'Edit Custom Field', 'treweler' ),
            'new_item'              => esc_html__( 'New Custom Field', 'treweler' ),
            'view_item'             => esc_html__( 'View Custom Field', 'treweler' ),
            'all_items'             => esc_html__( 'Custom Fields', 'treweler' ),
            'search_items'          => esc_html__( 'Search Custom Fields', 'treweler' ),
            'not_found'             => esc_html__( 'No Custom Fields found.', 'treweler' ),
            'not_found_in_trash'    => esc_html__( 'No Custom Fields found in Trash.', 'treweler' ),
            'menu_name'             => esc_html_x( 'Treweler', 'Admin menu name', 'treweler' ),
            'name_admin_bar'        => esc_html_x( 'Custom Field', 'Add New on Toolbar', 'treweler' ),
            'parent_item_colon'     => esc_html__( 'Parent Custom Fields:', 'treweler' ),
            'insert_into_item'      => esc_html__( 'Insert into custom field', 'treweler' ),
            'uploaded_to_this_item' => esc_html__( 'Uploaded to this custom field', 'treweler' ),
            'filter_items_list'     => esc_html__( 'Filter Custom Fields list', 'treweler' ),
            'items_list_navigation' => esc_html__( 'Custom Fields list navigation', 'treweler' ),
            'items_list'            => esc_html__( 'Custom Fields list', 'treweler' ),
          ],
          'description'         => esc_html__( 'Adds custom fields for markers.', 'treweler' ),
          'public'              => false,
          'publicly_queryable'  => false,
          'query_var'           => false,
          'show_ui'             => true,
          'show_in_menu'        => current_user_can( 'edit_treweler' ) ? 'treweler' : true, //'edit.php?post_type=map',
          'show_in_nav_menus'   => false,
          'capability_type'     => 'treweler',
          'has_archive'         => false,
          'hierarchical'        => false,
          'can_export'          => true,
          'menu_position'       => null,
          'rewrite'             => false,
          'exclude_from_search' => true,
          'supports'            => [ 'title' ],
        ]
      )
    );

    register_post_type(
      'twer-templates',
      apply_filters(
        'treweler_register_post_type_templates',
        [
          'labels'              => [
            'name'                  => esc_html_x( 'Templates', 'Post type general name', 'treweler' ),
            'singular_name'         => esc_html_x( 'Template', 'Post type singular name', 'treweler' ),
            'add_new'               => esc_html__( 'Add New', 'treweler' ),
            'add_new_item'          => esc_html__( 'Add New Template', 'treweler' ),
            'edit_item'             => esc_html__( 'Edit Template', 'treweler' ),
            'new_item'              => esc_html__( 'New Template', 'treweler' ),
            'view_item'             => esc_html__( 'View Template', 'treweler' ),
            'all_items'             => esc_html__( 'Templates', 'treweler' ),
            'search_items'          => esc_html__( 'Search Templates', 'treweler' ),
            'not_found'             => esc_html__( 'No Templates found.', 'treweler' ),
            'not_found_in_trash'    => esc_html__( 'No Templates found in Trash.', 'treweler' ),
            'menu_name'             => esc_html_x( 'Treweler', 'Admin menu name', 'treweler' ),
            'name_admin_bar'        => esc_html_x( 'Template', 'Add New on Toolbar', 'treweler' ),
            'parent_item_colon'     => esc_html__( 'Parent Templates:', 'treweler' ),
            'insert_into_item'      => esc_html__( 'Insert into Template', 'treweler' ),
            'uploaded_to_this_item' => esc_html__( 'Uploaded to this Template', 'treweler' ),
            'filter_items_list'     => esc_html__( 'Filter Templates list', 'treweler' ),
            'items_list_navigation' => esc_html__( 'Templates list navigation', 'treweler' ),
            'items_list'            => esc_html__( 'Templates list', 'treweler' ),
          ],
          'description'         => esc_html__( 'Adds templates for markers.', 'treweler' ),
          'public'              => false,
          'publicly_queryable'  => false,
          'query_var'           => false,
          'show_ui'             => true,
          'show_in_menu'        => current_user_can( 'edit_treweler' ) ? 'treweler' : true,
          'show_in_nav_menus'   => false,
          'capability_type'     => 'treweler',
          'has_archive'         => false,
          'hierarchical'        => false,
          'can_export'          => true,
          'menu_position'       => null,
          'rewrite'             => false,
          'exclude_from_search' => true,
          'supports'            => [ 'title' ],
        ]
      )
    );

    register_post_type(
      'treweler-import',
      apply_filters(
        'treweler_register_post_type_import',
        [
          'labels'              => [
            'name'                  => esc_html_x( 'Import', 'Post type general name', 'treweler' ),
            'singular_name'         => esc_html_x( 'Import', 'Post type singular name', 'treweler' ),
            'add_new'               => esc_html__( 'Add New', 'treweler' ),
            'add_new_item'          => esc_html__( 'Add New Import', 'treweler' ),
            'edit_item'             => esc_html__( 'Edit Import', 'treweler' ),
            'new_item'              => esc_html__( 'New Import', 'treweler' ),
            'view_item'             => esc_html__( 'View Import', 'treweler' ),
            'all_items'             => esc_html__( 'Import', 'treweler' ),
            'search_items'          => esc_html__( 'Search Imports', 'treweler' ),
            'not_found'             => esc_html__( 'No Imports found.', 'treweler' ),
            'not_found_in_trash'    => esc_html__( 'No Imports found in Trash.', 'treweler' ),
            'menu_name'             => esc_html_x( 'Treweler', 'Admin menu name', 'treweler' ),
            'name_admin_bar'        => esc_html_x( 'Import', 'Add New on Toolbar', 'treweler' ),
            'parent_item_colon'     => esc_html__( 'Parent Imports:', 'treweler' ),
            'insert_into_item'      => esc_html__( 'Insert into import', 'treweler' ),
            'uploaded_to_this_item' => esc_html__( 'Uploaded to this import', 'treweler' ),
            'filter_items_list'     => esc_html__( 'Filter Imports list', 'treweler' ),
            'items_list_navigation' => esc_html__( 'Imports list navigation', 'treweler' ),
            'items_list'            => esc_html__( 'Imports list', 'treweler' ),
          ],
          'description'         => esc_html__( 'Adds the ability to import markers.', 'treweler' ),
          'public'              => false,
          'publicly_queryable'  => false,
          'query_var'           => false,
          'show_ui'             => true,
          'show_in_menu'        => current_user_can( 'edit_treweler' ) ? 'treweler' : true, //'edit.php?post_type=map',
          'show_in_nav_menus'   => false,
          'capability_type'     => 'treweler',
          'has_archive'         => false,
          'hierarchical'        => false,
          'can_export'          => true,
          'menu_position'       => null,
          'rewrite'             => false,
          'exclude_from_search' => true,
          'supports'            => [ 'title' ],
        ]
      )
    );
  }

  /**
   * Customize taxonomies update messages.
   *
   * @param $messages
   * @since 1.14
   *
   * @return mixed
   */
  public static function updated_term_messages( $messages ) {
    $messages['map-category'] = array(
      0 => '',
      1 => __( 'Category added.', 'treweler' ),
      2 => __( 'Category deleted.', 'treweler' ),
      3 => __( 'Category updated.', 'treweler' ),
      4 => __( 'Category not added.', 'treweler' ),
      5 => __( 'Category not updated.', 'treweler' ),
      6 => __( 'Categories deleted.', 'treweler' ),
    );

    return $messages;
  }

  /**
   * Disable Gutenberg for custom post types.
   *
   * @param $postTypes
   * @param $can_edit
   * @param $post_type
   *
   * @return string[]
   */
  public static function gutenberg_can_edit_post_type( $postTypes, $can_edit, $post_type ) {
    return wp_parse_args( [ 'twer-templates', 'twer-custom-fields', 'twer-shapes', 'treweler-import' ], $postTypes );
  }

}

TWER_Post_Types::init();
