<?php
/**
 * Pro Treweler Map function
 * Functions for the map system.
 *
 * @package  Treweler\Functions
 * @version  1.14
 */

defined( 'ABSPATH' ) || exit;


if ( ! function_exists( 'twerGetMapShapes' ) ) {
	/**
	 * Retrieve screen shapes based on the provided map ID.
	 * This function fetches shapes associated with a given map ID
	 * from the 'twer-shapes' custom post type. It specifically targets
	 * posts that have a '_treweler_map_id' meta key matching the given
	 * map ID or its serialized version.
	 *
	 * @param int $mapId The ID of the map to fetch associated shapes for.
	 *
	 * @return array An array of shape styles associated with the provided map ID. Empty array if none found.
	 */
	function twerGetMapShapes( int $mapId ): array {
		global $wpdb;

		$screen_shapes = [];

      $prepared_query = $wpdb->prepare( "
        SELECT p.ID, pm.meta_value as shape_styles
        FROM {$wpdb->prefix}posts p
        LEFT JOIN {$wpdb->prefix}postmeta pm 
        ON p.ID = pm.post_id 
        AND pm.meta_key = '_treweler_shape_styles'
        LEFT JOIN {$wpdb->prefix}postmeta pm2
        ON p.ID = pm2.post_id 
        AND pm2.meta_key = '_treweler_map_id'
        WHERE p.post_type = 'twer-shapes'
        AND p.post_status = 'publish'
        AND (
            (pm2.meta_value = %s)
            OR
            (pm2.meta_value LIKE %s)
        )", $mapId, '%' . $wpdb->esc_like( serialize( (string) $mapId ) ) . '%' );

      $results = $wpdb->get_results( $prepared_query, ARRAY_A );

		if ( $results ) {
			foreach ( $results as $row ) {
				if ( ! empty( $row['shape_styles'] ) ) {
					$screen_shapes[] = $row['shape_styles'];
				}
			}
		}

		return $screen_shapes;
	}
}
